pro S7_create_supersky, Event, group_name_vector, hwp_list_n, hwp_file_list, $
						file_names, scale_factors, image_group_number, $
						LED_wid, image_colors, $
						double_precision_flag, disk_mode, memory_mode, disk_PTR, $
						max_hwp
;
;	for group_name, accepts vectors pointing to HWP sorted image numbers,
;	file names, and computes supersky images for each HWP
;	and writes them to disk for fetching and use
;
;	LED_wid is the widget ID of the LED stack to write to
;	image_colors is the colors array for the LED stack
;
;	if disk_mode = 1, writes files to disk
;   if memory_mode = 1, passes pointer to heap location of supersky images
;						back to calling program
;
;	image_group_number contains the group number for each image (in file_names order)
;
;	DPC	20100531	moved relevant code from S7_group_coadding
;	    20100816	added test (and stop) for any supersky image that is zeroed due to lack of input images
;						(actually, stop on any for which N_images < 2)
;		20100825	added call to routine to remove stars before stacking (replacing with local modeled sky)
;		20100827	if can't build supersky (too few input images for that HWP), set value of supersky to 1
;						to bypass its correction in the coadders later
;
WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
s_ptr = PTR_struct.PTR_summary
w_ptr = PTR_struct.PTR_wid_id
g_ptr = PTR_struct.PTR_group
;
start_memory = MEMORY()
start_memory = start_memory[0]
;
;max_hwp = 15					; normal is 15 - use 0 for test purposes, only
;
supersky_mode = 'subtract'										; subtract (sky) or divide (2nd order FF)
;	supersky_mode = 'divide'
;
smooth_supersky_flag = (*w_ptr).S7_smooth_supersky_mode				; for post averaging smoothing
smooth_kernel = (*w_ptr).S7_box_width_value							; boxcar smoothing size
;
if((*w_ptr).S7_skip_RC_flag eq 1) then begin						; skip row, col flattening in model_background if set
	rowf = 1
	colf = 1
endif else begin
	rowf = 0
	colf = 0
endelse
;
; invoke classifier for separating stars from sky? (1=use separator; 0 = no)
;
ss_star_sky_flag = (*w_ptr).S7_ss_star_sky_flag
;star_sky_flag = 0                           ; to force this to not distinguish for superskies
;
astro_dir = (*s_ptr).analysis_dir + 'S3_Astrometry' + PATH_SEP()
;
print_text, Event, 'Starting SuperSky image calculation with disk, memory modes = '+$
					STRING(disk_mode, memory_mode, format='(I,2x,I)')
;
;	create supersky directory, if not already present
;
if(disk_mode eq 1) then begin
	output_dir = (*s_ptr).analysis_dir + 'S7_Coadding' + PATH_SEP() + 'Supersky_Images' + PATH_SEP()
	test = FILE_INFO(output_dir)
	if(NOT(test.exists) or NOT(test.directory)) then begin
		FILE_MKDIR, output_dir
		print_text, Event, 'Creating new directory: ' +  output_dir
	endif
endif
;
;	for each group in the list vector
;
n_groups = N_ELEMENTS(group_name_vector)
;
for igroup = 0, n_groups-1 do begin
	;
	;	set all the HWP LEDs to red
	;
	image_colors[*] = "red"
	WCT_LED_stack, LED_wid, image_colors
	;
	group_name = group_name_vector[igroup]
	print_text, Event, 'Computing SuperSky Images for group ' + group_name
	;
	supersky = PTR_NEW(fltarr(1024,1026,max_hwp+1))
	;
	;	for each HWP angle
	;
	for ihwp = 0, max_hwp do begin
;	for ihwp = 1, 1 do begin
		;
		;	set LED1 to gold-ish
		;
		this_LED = ihwp
		image_colors[this_LED] = "goldenrod light"
		WCT_LED_stack, LED_wid, image_colors
		;
		super_sky_image = PTR_NEW(fltarr(1024,1026))
		;
		;	if only one group, this is easier
		;
		if(n_groups eq 1) then begin
			this_n_files = hwp_list_n[ihwp]	   ; the number of files at this HWP angle
			if(this_n_files eq 0) then begin
				;
				;	no files to make super sky out of, skip this image
				;
				print_text, Event, "No images for hwp = "+STRING(ihwp,format='(I)')+" so no supersky computed"
				goto, skip_super
			endif
			;
			this_file_list = hwp_file_list[ihwp,0:this_n_files-1]		; list of filenames for this HPW angle
			this_file_names = file_names[this_file_list]
		endif else begin
			;
			;	if > 1 group, must parse a bit more
			;
			ind_group = WHERE(image_group_number eq igroup, n_in_group)
			if(n_in_group eq 0) then begin
				print_text, Event, "ERROR - no images in group " + STRING(igroup)
				stop
			endif
			;
			;	how many in this HWP?
			;
			full_hwp_list = REFORM(hwp_file_list[ihwp,0:hwp_list_n[ihwp]-1])		; list of all the hwp image numbers for this hwp
			;
			;	how many in this group?
			;
			ind_group = WHERE(image_group_number[full_hwp_list] eq igroup, n_group_hwp)
			if(n_group_hwp ne 0) then begin
				this_file_names = file_names[full_hwp_list[ind_group]]
				this_file_list = full_hwp_list[ind_group]
				this_n_files = N_ELEMENTS(this_file_names)
			endif
			;
			if(n_group_hwp lt 3) then begin
				print_text, Event, 'WARNING - group ' + group_name + ' had too few images to compute supersky for HWP = '+$
							STRING(ihwp) + ', skipping'
				(*super_sky_image)[*,*] = 1.0
				goto, re_enter
			endif
			;
		endelse
		;
		;	for every image, compute median sky value
		;
		sky_vec = fltarr(this_n_files)
		image_path = strarr(this_n_files)
		;
		for i_file = 0, this_n_files-1 do begin
			parts = STRSPLIT(FILE_BASENAME(this_file_names[i_file]),"_phot.dat",/REGEX,/EXTRACT)
			image_path[i_file] = astro_dir + parts[0] + '.fits'
			;
			;print_text, Event, '   '+image_path[i_file]
			;
			ptr_input_image = PTR_NEW(readfits(image_path[i_file],header,/silent))
			;
			;	squash LLQ if LLQ flag set
			;
			(*ptr_input_image) = llq_clear( (*ptr_input_image), header)
			;
			(*ptr_input_image) = fix_low_pixels( (*ptr_input_image), /TPS)
			;
			(*ptr_input_image) = fix_high_pixels((*ptr_input_image))
			;
			;	model background, passing pointer in and out
			;
			ptr_output_image = model_background(ptr_input_image, SKY=this_sky, SIGMA=this_sigma, $
													NOROWF=rowf, NOCOLF=colf, /LOW_ORDER)
			PTR_FREE, ptr_input_image
			;
			sky_vec[i_file] = this_sky
		endfor
		;
		zz = median_filtered_mean(sky_vec)
		mean_sky = zz[0]
		mean_sky_sigma = fltarr(5)
		mean_sky_sigma[0] = zz[1] 			; push down rms a bit more
		;
		;	and compute dispersion *across* portions of one image, use the smallest of all methods
		;
		; ul quad
		zsi = median_filtered_mean((*ptr_output_image)[400:500,550:650])
		mean_sky_sigma[1] = zsi[1]
		; ll quad
		zsi = median_filtered_mean((*ptr_output_image)[400:500,400:500])
		mean_sky_sigma[2] = zsi[1]
		;
		;	if LLQ flag set, set this to a high value
		;
		LLQ_flag = SXPAR(header, 'PP3_LLQF', count=kwcount)
		if(kwcount eq 1) then begin
			if(LLQ_flag eq 1) then begin
				mean_sky_sigma[2] = 10000.0
			endif
		endif
		;
		; ur quad
		zsi = median_filtered_mean((*ptr_output_image)[550:650,550:650])
		mean_sky_sigma[3] = zsi[1]
		; lr quad
		zsi = median_filtered_mean((*ptr_output_image)[550:650,400:500])
		mean_sky_sigma[4] = zsi[1]
		;
		PTR_FREE, ptr_output_image
		;
		save_mean_sky_sigma = mean_sky_sigma
		mean_sky_sigma = MIN(mean_sky_sigma)
		;
		if(mean_sky_sigma eq 0.0) then begin
			print_text,Event,'mean sky sigma values = '+STRING(save_mean_sky_sigma)
			stop
		endif
		;
		;	then, in pieces, remove skies, compute median image
		;
		nslice = CEIL(this_n_files / 16)
		if(max_hwp eq 0) then nslice = CEIL(this_n_files / 8)
		nslice >= 1
		if(nslice gt 1) then print_text, Event, 'Computing median in '+STRING(nslice)+' image slices'
		;
		dy = FLOOR(1026 / nslice)
		;
		for islice = 0, nslice-1 do begin
			y0 = 0 + islice * dy
			y1 = y0 + (dy - 1)
			if(islice eq (nslice-1)) then y1 = 1025
			;
			if(nslice gt 1) then print_text, Event, 'Reading images for slice number '+$
									STRING(islice+1)+' (y0,y1) = '+STRING(y0,y1)
			;
			ptr_raw_images = PTR_NEW(fltarr(this_n_files,1024,(y1-y0+1)))
			;
			if(ss_star_sky_flag eq 1) then sigma_scale = 2.5 else sigma_scale = 1000.0       ; if set, allow higher sigma for stars
			;
			for i_file = 0, this_n_files-1 do begin
;			for i_file = 30, 30 do begin
				ptr_this_image = PTR_NEW(readfits(image_path[i_file],header,/silent))
				;
				(*ptr_this_image) = llq_clear( (*ptr_this_image), header)
				;
				(*ptr_this_image) = fix_low_pixels( (*ptr_this_image), /TPS)
				;
				(*ptr_this_image) = fix_high_pixels((*ptr_this_image))  ; to remove data transfer junk
				;
;				print, 'i_file = ',i_file,' image_path = ',FILE_BASENAME(image_path[i_file])
				;
				(*ptr_raw_images)[i_file,*,0:(y1-y0)] = S7_mask_stars((*ptr_this_image)[*,y0:y1],SIGMA_SCALE = sigma_scale)
				;
				PTR_FREE, ptr_this_image
				;
			endfor
			;
			;	read each image into the input array
			;
			new_axis1 = 1024
			new_axis2 = (y1 - y0) + 1
			axis1 = 1024
			axis2 = (y1 - y0) + 1
			n_images = this_n_files
			x_off = fltarr(this_n_files)		; this sets all offsets to 0 so we can eliminate stars
			y_off = fltarr(this_n_files)		; ditto
			scale = scale_factors[this_file_list]
			sky_image = sky_vec
			;
			scaled_mean_sky = mean_sky
			debug = 0
			mean_images = 1.0
			;
			;	20100816 - if n_images le 2, stop
			;
			if(n_images le 2) then begin
				print_text, Event, "ERROR - too few images to compute supersky"
				print_text, Event, "... HWP = "+STRING(ihwp,format='(I)')+" n_images = "+STRING(n_images,format='(I)')
				if(n_images le 1) then begin
					print, "Warning - too few images to compute supersky for HWP ",ihwp,$
								' of group ',group_name
				endif
			endif
			;
			if(ss_star_sky_flag eq 1) then mean_sky_sigma = mean_sky_sigma else mean_sky_sigma = 1000.0*mean_sky_sigma
			;
			coadd_struct = { new_naxis1:		new_axis1,	$	x-axis size of output image
							 new_naxis2: 		new_axis2, 	$	y-axis size of output image
							 naxis11:			axis1-1, 	$	x-axis size of input images, minus 1
							 naxis21:			axis2-1, 	$	y-axis size of input images, minus 1
							 n_images:			n_images, 	$	number of constituent images
							 x_off:				x_off,		$	vector of x offsets
							 y_off:				y_off, 		$	vector of y offsets
							 raw_images:		(*ptr_raw_images),	$	the input data cube
							 sky_image:			sky_image,	$	vector of sky means to subtract from each image
							 scale:				scale,		$	multiplicative scale factor for each image
							 double_precision_flag: double_precision_flag, $
							 fn_mean_images: 	mean_images, $ number of images scaled to
							 mean_sky_sigma: 	mean_sky_sigma, $ uncertainty in sky value
							 debug: 			debug, $
							 scaled_mean_sky:	scaled_mean_sky }

			ptr_output_image = S7_coadd(coadd_struct)
			;
			(*super_sky_image)[*,y0:y1] = (*ptr_output_image)[*,*]
			;
			print_text, Event, group_name + ' HPW'+STRING(ihwp+1,format='(i2)')+' SuperSky computed. Mean, sigma sky = '+STRING(mean_sky, mean_sky_sigma)
			;
			PTR_FREE, ptr_output_image, ptr_raw_images
		endfor      ; end of slice loop
		;
		;	mask, then normalize by mean sky, saving deleted data (turn to 1.0)
		;
		mask = PTR_NEW(fltarr(1024,1026))
		*mask = (((*super_sky_image + 1.0e5) > 0.0) < 1.0)		; all deleted turned off, others on
		*super_sky_image /= mean_sky
		*super_sky_image *= *mask
		ind = WHERE(*mask eq 0.0,n_ind)
		if(n_ind gt 0) then (*super_sky_image)[ind] = 1.0
		;
		PTR_FREE, mask
		;
		;	median filter
		;
		re_enter:
		;
		; if smooth_kernel > 1 then median filter
		;
		if(smooth_supersky_flag eq 1 and (smooth_kernel gt 1 and smooth_kernel lt 100)) then begin
		    print_text, Event, "performing median filtering on SuperSky with kernel = "+STRING(smooth_kernel)+" pixels"
		    (*supersky)[*,*,ihwp] = MEDIAN(*super_sky_image, smooth_kernel)
		endif else begin
		    print_text, Event, "no median filtering on SuperSky images"
				(*supersky)[*,*,ihwp] = (*super_sky_image)
		endelse
		;
		;stop
		;
		PTR_FREE, super_sky_image
		;
		goto, next_hwp
		;
		skip_super:
		(*supersky)[*,*,ihwp] = 1.0		; turns off correction for this hwp
		;
		next_hwp:
		;
		;	before looping, be sure to close supersky_image from heap
		;
		type = SIZE(super_sky_image, /TYPE)
		if(type eq 10) then begin
			PTR_FREE, super_sky_image
		endif
		;
	endfor     ; HWP loop end
	;
	;	depending on mode, return pointer or write superskys to disk
	;
	disk_PTR = supersky
	;
	if(disk_mode eq 0) then begin
		goto, finish
	endif
	;
  ;   write to disk selected, do so
	;
	for ihwp = 0, max_hwp do begin
;	for ihwp = 1, 1 do begin
		;
		output_name = 'Supersky_' + replace_underscores(group_name) + "_HWP_" + STRTRIM(STRING(ihwp,format='(I2.2)'),2) + '.fits'
		output_dir = (*s_ptr).analysis_dir + 'S7_Coadding' + PATH_SEP() + 'Supersky_Images' + PATH_SEP()
		output_path = output_dir + output_name
		;
		writefits, output_path, (*supersky)[*,*,ihwp]
    ;
		print_text, Event, "Wrote Super-Sky image "+FILE_BASENAME(output_path)+" to disk"
		;
		;	set LED1 to light-blue
		;
		this_LED = ihwp
		image_colors[this_LED] = "skyblue"
		WCT_LED_stack, LED_wid, image_colors
		;
		;	compute mean sky value (test)
		;
	endfor
	;
	if(disk_mode eq 0) then begin
		PTR_FREE, supersky
	endif
	;
	finish:
	print_text, Event, 'Within S7 create supersky - do heap garbage collection'
	HEAP_GC, /PTR
endfor
;
;	before departing, be sure to close supersky_image from heap
;
type = SIZE(super_sky_image, /TYPE)
if(type eq 10) then begin
	PTR_FREE, super_sky_image
endif
;
;	check memory usage
;
end_memory = MEMORY()
end_memory = end_memory[0]
dif_memory = end_memory - start_memory
if(dif_memory gt 15000000L and memory_mode eq 0) then begin
	print_text, Event, 'Memory Leak - Warning'
;	stop
endif
;
print_text, Event, 'SuperSky image computations finished'
print_text, Event, '  Starting Heap Garbage Collection '
;
HEAP_GC, /PTR
;
end
