pro clean_PSF, PSF_header, PSF_residuals, DEBUG=debug
;
;	takes PSF model, computes radial intensity
;	and masks all residuals pixels fainter than
;	mag_cut mag below peak
;
;	DPC	20100305
;
if(~arg_present(DEBUG)) then debug = 0
;
residuals = double(PSF_residuals)
header = PSF_header
;
;mag_cut = 6.0					; default value
mag_cut = 10.0					; increased to better get skirts of bright stars - 20120531 DPC
;
;----------------------------------------------------------------
gauss_gain = SXPAR(header, 'GAUSS1')
gauss_x0 = SXPAR(header, 'GAUSS2')
gauss_y0 = SXPAR(header, 'GAUSS3')
gauss_dx = SXPAR(header, 'GAUSS4')
gauss_dy = SXPAR(header, 'GAUSS5')
;
psfmag = SXPAR(header, 'PSFMAG')
fit_rad = SXPAR(header, 'FITRAD')
psfrad = SXPAR(header, 'PSFRAD')
;
gauss_fit = [gauss_gain, gauss_x0, gauss_y0, gauss_dx, gauss_dy]
;----------------------------------------------------------------
;
if(debug eq 1) then begin
	;
	;	compute reconstructed PSF using DAOPHOT code
	;
	g_remake_psf, gauss_fit, PSF_residuals, psfrad, psf_image, psfmag, fit_rad
	;
	file_psf = dialog_pickfile(title='PSF reconstruct ala DAOPHOT')
	writefits, file_psf, psf_image
endif
;

isize = SIZE(residuals)
nx = isize[1]
ny = isize[2]
;
center = FLOOR(nx/2)
x_pix = (findgen(nx) - center) * 0.5		; for resid array, which is 2x zoomed
y_pix = (findgen(ny) - center) * 0.5
;
gauss = dblarr(nx, ny)
;
;	rebuild gaussian
;
for iy = 0, ny-1 do begin
	y_power = (y_pix[iy] - gauss_y0) / gauss_dy
	y_num = y_power * y_power / 2.0
	if (y_num lt 14.0) then begin
		for ix = 0, nx-1 do begin
			x_power = (x_pix[ix] - gauss_x0)/gauss_dx
			x_num = x_power * x_power / 2.
			if (x_num lt 14.0) then begin
				;
				;	split into 10x10 grid
				;
				gauss[ix,iy] = 0.0
				for ky = 0, 9 do begin
					little_y = y_pix[iy] + (float(ky) - 4.5) / 10.0
					y_power = (little_y - gauss_y0) / gauss_dy
					y_num = y_power * y_power / 2.0
					if(y_num lt 14.0) then begin
						for kx = 0, 9 do begin
							little_x = x_pix[ix] + (float(kx) - 4.5) / 10.0
							;
							x_power = (little_x - gauss_x0) / gauss_dx
							x_num = x_power * x_power / 2.0
							if(x_num lt 14.0) then begin
								;
								;	finally, evaluate the gaussian
								;
								gain = gauss_gain * exp(-1.0*(y_num + x_num)) / 100.0
								gauss[ix,iy] += gain
							endif
						endfor
					endif
				endfor
			endif else begin
				gauss[ix,iy] = 0.0d0
			endelse
		endfor
	endif else begin
		 gauss[*,iy] = 0.0d0
	endelse
endfor
;
;	combine gaussian with residuals
;
image = residuals + gauss
;
if(debug eq 1) then begin
	file_out = dialog_pickfile(Title='recreated PSF image')
	writefits, file_out, image, header
endif
;
;	find radial intensity
;
radius = fltarr(nx,ny)
inten = dblarr(nx,ny)
integrated = dblarr(nx * ny)
;
i = 0L
for iy = 0, ny-1 do begin
	for ix = 0, nx-1 do begin
		r = (x_pix[ix])^2 + (y_pix[iy])^2
		radius[ix,iy] = sqrt(r)
		inten[ix,iy] = image[ix,iy]
		i++
	endfor
endfor
;
ind_rad = SORT(radius)
;
scale = 0.58				; true scale in arc sec per Mimir pixel (not zoomed pixel)
radius *= scale
;
max_inten = MAX(inten)
inten /= max_inten
mag_inten = -2.5*alog10(inten > 1.0e-10)
;
;	compute integrated power
;
ntot = nx * ny
for i = 0, ntot-1 do begin
	this_i = ind_rad[i]
	if(i eq 0) then begin
		integrated[this_i] = inten[this_i]
	endif else begin
		integrated[this_i] = inten[this_i] + integrated[ind_rad[i-1]]
	endelse
endfor
;
;	max integrated
;
max_integ = MAX(integrated[ind_rad],ind_max)
integrated /= max_integ
;
scaled = 1000.0*(integrated-1.0)
scaled += 1.0
;
mag_scaled = -2.5*alog10(scaled > 1.0e-10)
;
;	remove pixels for which the intensity is > mag_cut mag under peak
;
ind_mag = WHERE(mag_inten gt mag_cut or ~FINITE(mag_inten), n_mag)
if(n_mag gt 0) then begin
	if(debug eq 1) then print, "number of pixels with mag > mag_cut = ",n_mag
	image[ind_mag] = 0.0d0
endif
;
;	zero image for all radii outside the max integrated
;
rad_max = radius[ind_rad[ind_max]]
;
ind_out = WHERE(radius gt rad_max and image ne 0.0d0, n_out)
if(n_out gt 0) then begin
	if(debug eq 1) then print, "number of pixels with radii > r_max (",rad_max,") - 1.5 pix = ",n_out
	image[ind_out] = 0.0d0
endif
;
;	remove negative pixels, too
;
ind_neg = WHERE(image lt 0.0 or ~FINITE(image), n_neg)
if(n_neg gt 0) then begin
	if(debug eq 1) then print, "number of pixels still having negative values = ",n_neg
	image[ind_neg] = 0.0
endif
;
;	compute mean radius to mag_cut to mag_cut+0.5
;
ind_mean_r = WHERE(mag_inten ge mag_cut and mag_inten lt (mag_cut + 0.5),n_mean_r)
if(n_mean_r gt 2) then begin
	zz = median_filtered_mean(radius[ind_mean_r])
	mean_r = zz[0]
	ind_r = WHERE(radius gt mean_r, n_r)
	if(n_r gt 0) then begin
		image[ind_r] = 0.0
	endif
endif
;
;	create mask for residuals
;
mask = (long(image) < 1) > 0
residuals *= mask
;
;	and write out
;
if(debug eq 1) then begin
	file_out = dialog_pickfile(title='New PSF fits file')
	writefits, file_out, residuals, header
endif
;
if(debug eq 1) then begin
	device, decomposed=0
	tvlct, [0,255,255,0], [0,255,0,0], [0,255,0,255], 0
	black = 0
	white = 1
	red=2
	blue=3
	;
	window, 15, xsize=500, ysize=500, title='PSF_Model: Intensity vs Radius'
	;
	plot, radius[ind_rad], mag_inten[ind_rad], xr = [0,max(radius)],yr = [10,-1],$
			xstyle=1,ystyle=1,/nodata,$
			color=black,background=white, xtitle='Radial Offset [arcsec]',$
			ytitle='Normalized Intensity, 1000*Normalized Integrated'
	;
	oplot, radius[ind_rad], mag_inten[ind_rad], color=red, psym=2
	;
	oplot, radius[ind_rad], mag_scaled[ind_rad], color=blue, psym=-2, symsize=0
	;
	cursor, xx, yy, /wait
	wait, 0.3
	;
;	file_out = dialog_pickfile(Title='Modified PSF Image')
;	writefits, file_out, image, header
endif
;
;	return masked residuals
;
PSF_residuals = residuals
;
end