pro dpc_image_combine, input_list, output_average_image, str_kernel, $
          output_stdev_image, output_coverage_image, LONG_OUTPUT=long_output, $
          NO_SKY=no_sky
;
;   image combiner, ala IRAF imcombine
;
;   accepts list of images and performs summation,
;   producing a covereage-averaged image (not median filtered), a
;   median-filtered average image as
;   well as coverage and sigma images, if requested
;
;   if LONG_OUTPUT switch on, output standard deviation and coverage images will be stored
;
;   DPC 20041014
;       20041031     updated for Oct '04 image sizes
;       20051102     modified to allow command line arguments and to change to single file/offsets input file
;                    also, if sky removal selected, sky value removed is stored as comment in header
;       20060103     copied from imcombine.pro, and modified to use tp routines
;                          -- removed bad pixel masking stuff
;                          -- determine sky shape first
;                          -- determine image pixel shifts next
;                          -- removing sky shape from each image (scaled to image sky value), combine images
;                          -- clean final result with reset_high_low and fix_bad_pixels
;       20060106     removed cross-correlation code, replacing it with DAOPHOT star finding and matching to get offsets
;       20060107     reordered to reduce raw image I/O and processing
;                     changed summed image output to coverage-averaged with sky removed
;       20060128     tuned rejection criteria to be a bit less aggressive about rejecting images, added image name sort
;       20060803     modified to allow arbitrary sized input images (esp. the output images from a previous run)
;
;		20080906	 changed sign of offsets (why did this need to take place?)
;		20081025	 changed to get offsets from astrometry
;	    20081030	 looks for multiplicative gain factor in FITS header and applies if not already applied
;
;------------------------------------------------------------------------------
if(N_ELEMENTS(LONG_OUTPUT) eq 0) then LONG_OUTPUT = 0 else LONG_OUTPUT = 1
if(N_ELEMENTS(NO_SKY) eq 0) then NO_SKY = 0 else NO_SKY = 1
;
NO_SKY = 0			; 1 = no sky fit; 0 = planar fit
;
print, "Starting pol_image_combine_no_sky (v1); Long-output and No-sky switches set to: ",LONG_OUTPUT, NO_SKY
;
np = N_PARAMS()
;
reads, STRTRIM(str_kernel,2), kernel, format='(i)'
;
deleted_data = double(-1.0e6)
;
if(np eq 0) then begin
    image_list = dialog_pickfile(Title="Name of input list file of images")
    trial_name = FILE_DIRNAME(image_list,/MARK_DIRECTORY) + FILE_BASENAME(image_list,".dat") + "_sum.fits"
    out_mean = dialog_pickfile(Title="Name an Output Average Image",file=trial_name)
    iyn = ''
    print,"Shall Output Images of STDEV and Coverage be saved? (Y/N)"
    read, iyn
    if(STRUPCASE(iyn) eq 'Y') then LONG_OUTPUT=1
    if(LONG_OUTPUT eq 1) then begin
        trial_name = FILE_DIRNAME(image_list,/MARK_DIRECTORY) + FILE_BASENAME(image_list,".dat") + "_stdev.fits"
       out_sigma = dialog_pickfile(Title="Name an Output STDEV Image",file=trial_name)
       trial_name = FILE_DIRNAME(image_list,/MARK_DIRECTORY) + FILE_BASENAME(image_list,".dat") + "_cvg.fits"
       out_coverage = dialog_pickfile(Title="Name a, Output Coverage Image",file=trial_name)
    endif
endif else begin
    image_list = input_list
    out_mean = output_average_image
    if(LONG_OUTPUT eq 1) then begin
       out_sigma = output_stdev_image
       out_coverage = output_coverage_image
    endif
endelse
;
;   read input list of file names
;
openr, image_list_lun, image_list, /GET_LUN
path = ''
images = strarr(500)
nimages = 0L
line = ''
dx = 0.0
dy = 0.0
while (~EOF(image_list_lun)) do begin
    readf, image_list_lun, line
    images[nimages] = STRTRIM(line,2)
    nimages++
endwhile
FREE_LUN, image_list_lun
images = images[0:nimages-1]
;
;     sort the image names by image number (increasing) -- this should improve group matching
;
image_no = intarr(nimages)
;
;	set all image qualities to good for now
;
image_quality = intarr(nimages)
image_quality[*] = 1
;
image_gain = dblarr(nimages)		; multiplicative gain for each image
;
for ii = 0, nimages-1 do begin
    base_image = FILE_BASENAME(images[ii])
    ;
    ;  parse to get number
    ;
    parts = STRSPLIT(base_image,".",COUNT=dot_count,/EXTRACT)
    if(dot_count le 1) then goto, skip_sort
    rest = STRSPLIT(parts[1],"_LDFC",COUNT=ldfc_count,/EXTRACT)
    if(ldfc_count le 1) then goto, skip_sort
    reads, rest[0], frame_no
    image_no[ii] = frame_no
endfor
;
isort = SORT(image_no)
images = images[isort]  ;    put images into order
;
skip_sort:          ; skip the sort if any filenames don't match simple LDFC pattern
;----------------------------------------------------------------------------------------------
;
;   read and clean the images and save in an array
;
header = strarr(nimages,500)
hlines = intarr(nimages)
;
;   read all headers to discover largest image sizes
;
ncol = 0
nrow = 0
;
image_total_time = dblarr(nimages)
image_total_images = intarr(nimages)
image_exp_time = dblarr(nimages)
;
;	also compute offsets of each image relative to the first
;	using astrometry information to compute RA, Dec of (0,0)
;	pixel
;
offsetx = fltarr(nimages)
offsety = fltarr(nimages)
;
for ifile = 0, nimages-1 do begin
    this_header = HEADFITS(images[ifile])
    icol = FXPAR(this_header,"NAXIS1")
    irow = FXPAR(this_header,"NAXIS2")
    ;
    if(icol gt ncol) then ncol = icol
    if(irow gt nrow) then nrow = irow
    ;
    ;   look for keywords
    ;
    totltime = FXPAR(this_header,"TOTLTIME")
    if(!ERR ne 0) then begin
        !ERR = 0
        totltime = -1
    endif
    ;
    image_exp_time[ifile] = FXPAR(this_header,"EXPTIME")
    if(totltime ne -1) then image_total_time[ifile] = totltime else image_total_time[ifile] = image_exp_time[ifile]
    ;
    ncoadds = FXPAR(this_header,"TOTLIMAG")
    if(!ERR ne 0) then begin
        !ERR = 0
        ncoadds = 1
    endif
    image_total_images[ifile] = ncoadds
    ;
    ;	get RA, Dec of (0,0) pixel
    ;
    CRPIX1 = FXPAR(this_header, "CRPIX1")
    CRPIX2 = FXPAR(this_header, "CRPIX2")
    CRVAL1 = FXPAR(this_header, "CRVAL1")
    CRVAL2 = FXPAR(this_header, "CRVAL2")
    CD1_1 = FXPAR(this_header, "CD1_1")
    CD2_2 = FXPAR(this_header, "CD2_2")
    ;
    ;	correct scale factor in RA for Cos(dec)
    ;
    cos_dec = cos(CRVAL2 * (!DPI/180.0d0))
    ;
    CD1_1 = -double(0.5787) / 3600.0d0									; deg per pixel
    CD2_2 = -CD1_1
    ;
    first_RA = CRVAL1 - FLOAT(CRPIX1) * CD1_1 / cos_dec
    first_Dec = CRVAL2 - FLOAT(CRPIX2) * CD2_2
    ;
    if(ifile eq 0) then begin
    	ref_RA = first_RA
    	ref_Dec = first_Dec
    endif
    ;
    ;
    offsetx[ifile] = ROUND((first_RA - ref_RA)*(cos_dec / CD1_1))	; in pixels - these offsets must be added to the
    offsety[ifile] = ROUND((first_Dec - ref_Dec) / CD2_2)	;  indicated image to match the sky position of the first image
    ;
    ;	check for presence of gain keywords
    ;
    pol_gain = SXPAR(this_header, "POL_GAIN",COUNT=pcount)
    ;
    if(pcount ge 1) then begin
    	pgain_on = SXPAR(this_header, "PGAIN_ON",COUNT=pgcount)
    	if(pgcount ge 1) then begin
    		;
    		;	ok, both keywords present
    		;
    		if NOT(PGAIN_ON) then image_gain[ifile] = pol_gain else image_gain[ifile] = 1.0d0
    	endif else image_gain[ifile] = 1.0d0
    endif else image_gain[ifile] = 1.0d0
  	;
    print, "for image ",ifile," named ",FILE_BASENAME(images[ifile]),$
    	" offset X, Y = ",offsetx[ifile],offsety[ifile]," pixels and gain set to ",image_gain[ifile]
    ;
endfor
;
raw_data = dblarr(nimages, ncol, nrow)
raw_data[*,*,*] = deleted_data
;
for ifile = 0, nimages-1 do begin
    raw_image = readfits(images[ifile], this_header)
    hlines[ifile] = N_ELEMENTS(this_header)
    header[ifile,0:hlines[ifile]-1] = this_header
    ;raw_image = reset_high_low_pixels(raw_image)		removed 20081025 for better photometric quality
    ;raw_image = fix_bad_pixels(raw_image)				removed 20081025 for better photometric quality
    ;
    image_size = SIZE(raw_image)
    icol = image_size[1]
    irow = image_size[2]
    ;
    ;   locate this image in the lower left corner of the array
    ;
    ;		scale by gain, except where deleted data
    ;
    sc_image = raw_image[*,*]
    ind = WHERE(raw_image ne deleted_data,ngood)
    sc_image[ind] = sc_image[ind] * image_gain[ifile]
    ;
    ;	test...
    ;
    xnd = WHERE(raw_image eq deleted_data,nbad)
    if(nbad eq 0) then print, "Didn't find any deleted pixels - THIS SHOULD NOT HAPPEN!"
    ;
    print, "read image ",FILE_BASENAME(images[ifile])," and found ",ngood," good pixels and ",nbad," deleted pixels"
    ;
    raw_data[ifile,0:icol-1,0:irow-1] = sc_image[*,*]	; load array
endfor
;
;---------------------------------------------------------------------------------------------
;
; determine size of output image
;
ind_m = WHERE(offsetx ne 99999.0,m_count)
if(m_count ne 0) then begin
    max_x = round(max(offsetx[ind_m]))
    max_y = round(max(offsety[ind_m]))
    min_x = round(min(offsetx[ind_m]))
    min_y = round(min(offsety[ind_m]))
endif else begin
    print, "ERROR - no good images. Stopping"
    stop
endelse
;
xsize = ncol + (max_x - min_x)
ysize = nrow + (max_y - min_y)
;
;   reserve this much space for the output images
;
image_mean = dblarr(xsize, ysize)
image_MFM = dblarr(xsize,ysize)
;
if(LONG_OUTPUT eq 1) then begin
    image_sigma = fltarr(xsize, ysize)
endif
image_coverage = intarr(xsize, ysize)
;
;   and the array to hold all the images
;
;big_image = dblarr(nimages, xsize, ysize)
big_image_no_sky = dblarr(nimages, xsize, ysize)
big_image_no_sky[*,*,*] = deleted_data
;
;---------------------------------------------------------------------------
;
;   find mean sky shape (scaled to 100.0)
;
if(NO_SKY eq 0) then sky_100 = pol_sky_combine_sub(raw_data,image_quality,kernel)
;
;---------------------------------------------------------------------------
;
;   read in all the images into the big_image array, scaling sky to current image, and subtracting
;
sky = fltarr(nimages)
;
igood = 0
;
;   also, sum the total integration time and image count
;
total_time = 0.0
total_images = 0
;
for ifile = 0, nimages-1 do begin
    ;
    ;
    total_time = total_time + image_total_time[ifile]
    total_images = total_images + image_total_images[ifile]
    exp_time = image_exp_time[ifile]
    ;
    ;   save a copy of the first good image header as template for output header
    ;
    if(igood eq 0) then begin
        out_header = REFORM(header[ifile,0:hlines[ifile]-1])
        igood = 1   ; don't need the rest
    endif
    ;
    ;   determine sky in this image
    ;
    this_image = REFORM(raw_data[ifile,*,*])
    ;
    if(NO_SKY eq 0) then begin
        pol_find_sky, this_image, sky_mean, sky_sigma
        sky[ifile] = sky_mean
        ;
        ;   scale sky shape image to this mean
        ;
        this_sky = sky_100 * (sky_mean / 100.0) ;   sky shape image scale to current sky level
        ;
        ;   difference (don't correct any pixel at zero, as this is a special value for deleted regions)
        ;
        sky_removed_image = this_image * 0.0 + deleted_data
        ;
        ;ind_nz = WHERE(this_image ne 0.0, count_nz)
        ind_nz = WHERE(this_image ne deleted_data, count_nz)
        ;
        if(count_nz gt 0) then sky_removed_image[ind_nz] = this_image[ind_nz] - this_sky[ind_nz]
        ;
;        ind_sz = WHERE(sky_mean eq 0.0,count_sz)
;        if(count_sz gt 0) then sky_removed_image[ind_sz] = 0.
    endif else begin
        sky_removed_image = this_image
        sky_100 = 0.0
        sky_mean = 100.0
    endelse
    ;
    ;   pack resulting image into big_image, using the offsets and masking, if selected
    ;
    xoffset = round(offsetx[ifile] - min_x)
    yoffset = round(offsety[ifile] - min_y)
    ;
    for icol = 0, ncol-1 do begin
        for irow = 0, nrow-1 do begin
            big_image_no_sky[ifile,xoffset+icol,yoffset+irow] = sky_removed_image[icol,irow]
        endfor
    endfor
    ;
endfor
;
;   for every pixel in the final resulting images...
;
ind_good = WHERE(image_quality eq 1)
for icol = 0, xsize-1 do begin
    for irow = 0, ysize-1 do begin
       ;
       ;raw_vec = REFORM(big_image[*,icol,irow])
       raw_vec_no_sky = REFORM(big_image_no_sky[*,icol,irow])
       raw_vec_no_sky = raw_vec_no_sky[ind_good]   ; only good data
       ;ind = where(raw_vec_no_sky ne 0.0, ngood)       ; and that, nonzero
       ;
       ind = where(raw_vec_no_sky ne deleted_data, ngood)
       ;
       if(ngood eq 0) then begin
;            image_mean[icol,irow] = 0.0           ; seems better than deleted data...
;            image_MFM[icol,irow] = 0.0
;            image_coverage[icol,irow] = 0
            ;
            image_mean[icol,irow] = deleted_data           ; seems better than deleted data...
            image_MFM[icol,irow] = deleted_data
            image_coverage[icol,irow] = 0
            if(LONG_OUTPUT eq 1) then begin
                image_sigma[icol,irow] = 100.
            endif
       endif else begin
            ;
            ;     load the good data into a vector for median-filtering
            ;
            vector_no_sky = raw_vec_no_sky[ind]
            ;
            ;     compute median-filtered-mean and sigma
            ;
            mx = median_filtered_mean(vector_no_sky)     ; [0] is mean; [1] is sigma
            ;
            ;   use the following line to save the median filtered mean
            ;
            image_MFM[icol,irow] = mx[0]
            ;
            ;   for flux conservation, though, use summation over good pixels (w/i 5 sigma of mean)
            ;
            ind_tot = WHERE(abs(vector_no_sky - mx[0])/mx[1] le 5.,n_ind_tot)
            ;
            image_mean[icol,irow] = TOTAL(vector_no_sky[ind_tot])
            ;
            image_coverage[icol,irow] = n_ind_tot
            if(LONG_OUTPUT eq 1) then begin
                image_sigma[icol,irow] = mx[1]
            endif
      endelse
    endfor
endfor
;
;   normalize summed image by coverage, where non-zero
;
ind_z = WHERE(image_coverage eq 0, count_z)
if(count_z ne 0) then image_mean[ind_z] = deleted_data
ind_nz = WHERE(image_coverage ne 0, count_nz)
if(count_nz ne 0) then image_mean[ind_nz] = image_mean[ind_nz] / float(image_coverage[ind_nz])
;
;   compute median coverage and effective integration time
;
central_coverage = image_coverage[round(0.45*xsize):round(0.55*xsize),round(0.45*ysize):round(0.55*ysize)]
zz = median_filtered_mean(central_coverage)
;
ncoverage = round(zz[0])
;
;	move reference pixels over by -min_y and -min_x
;
CRPIX1 = FXPAR(this_header, "CRPIX1")
CRPIX2 = FXPAR(this_header, "CRPIX2")
;
CRPIX1 = CRPIX1 - min_x
CRPIX2 = CRPIX2 - min_y
;
FXADDPAR, out_header, "CRPIX1", CRPIX1, "Reference RA Pixel - pol_im_com"
FXADDPAR, out_header, "CRPIX2", CRPIX2, "Reference Dec Pixel - pol_im_com"
;
FXADDPAR, out_header, "EXPTIME", exp_time, "Time Reference for Count Rate Basis"
;
FXADDPAR, out_header, 'TOTLTIME', total_time,"Total Integration Time"
FXADDPAR, out_header, 'TOTLIMAG', total_images,"Total Images in Average"
;
;   done computing image, update header before writing
;
;   date/time
;   filename
;   method/software used
;   input images, with sky (if SKY=1)
;
stime = SYSTIME()
FXADDPAR, out_header, 'HISTORY',STRING(format='("Produced by    POL_IMAGE_COMBINE on ",a)',stime)
FXADDPAR, out_header, 'FILENAME', FILE_BASENAME(out_mean),"Coverage-Averaged Image"
;
;   average sky removed
;
no_sky_header = out_header
for ifile = 0, nimages-1 do begin
    if(image_quality[ifile] eq 1) then begin
        scomment1 = STRING(format='(";X,Y Off=",f7.2,",",f7.2)',offsetx[ifile],offsety[ifile])
        scomment2 = STRING(format='(";Sky= ",f7.2)',sky[ifile])
        scomment = scomment1+scomment2
        ;scomment = ' sky not removed'
        FXADDPAR, out_header, "COMMENT",STRING(format='("Included ",a)',FILE_BASENAME(images[ifile]))+scomment
        FXADDPAR, out_header, "COMMENT",STRING(format='(20x,"with applied gain = ",f5.3)',image_gain[ifile])
    endif else begin
        FXADDPAR, out_header, "COMMENT",STRING(format='("Image ",a," was rejected from inclusion")',FILE_BASENAME(images[ifile]))
    endelse
endfor
;
;	add gain info
;
FXADDPAR, no_sky_header, 'POL_GAIN', 1.0d0, "Effective gain reset to unity in coadd"
FXADDPAR, no_sky_header, 'PGAIN_ON', 'T', "Gain effectively applied"
;
writefits, out_mean, image_mean, out_header
;
;   now, also write the median-filtered-mean average image
;
for ifile = 0, nimages-1 do begin
    if(image_quality[ifile] eq 1) then begin
    	scomment1 = STRING(format='(";X,Y Off=",f7.2,",",f7.2)',offsetx[ifile],offsety[ifile])
        scomment2 =	STRING(format='(";Sky= ",f7.2)',sky[ifile])
        scomment = scomment1+scomment2
        ;scomment = STRING(format='("; Removed sky = ",f7.2)',sky[ifile])
        ;scomment = ' sky not removed'
        FXADDPAR, no_sky_header, "COMMENT",STRING(format='("Included ",a)',FILE_BASENAME(images[ifile]))+scomment
        FXADDPAR, no_sky_header, "COMMENT",STRING(format='(20x,"with applied gain = ",f5.3)',image_gain[ifile])
    endif else begin
        FXADDPAR, no_sky_header, "COMMENT",STRING(format='("Image ",a," was rejected from inclusion")',FILE_BASENAME(images[ifile]))
    endelse
endfor
out_mfm = FILE_DIRNAME(out_mean,/MARK_DIRECTORY) + FILE_BASENAME(out_mean,".FITS") + "_MFM.FITS"
FXADDPAR, no_sky_header, 'FILENAME', FILE_BASENAME(out_mean),"Median-filtered-mean average Image"
;
;	add gain info
;
FXADDPAR, no_sky_header, 'POL_GAIN', 1.0d0, "Effective gain reset to unity in coadd"
FXADDPAR, no_sky_header, 'PGAIN_ON', 'T', "Gain effectively applied"
;
;   get current integration time per image
;
;FXADDPAR, no_sky_header, 'EXPTIME', t_int, 'Average Integration Time'
writefits, out_mfm, image_MFM, no_sky_header
;
;
if(LONG_OUTPUT eq 1) then begin
    ;
    ;  update filename and comment
    ;
    FXADDPAR, out_header, 'FILENAME', FILE_BASENAME(out_sigma),"Standard Deviation Image"
    writefits, out_sigma, image_sigma, out_header
    ;
    FXADDPAR, out_header, 'FILENAME', FILE_BASENAME(out_coverage),"Coverage Image"
    writefits, out_coverage, image_coverage, out_header
endif
;
theend:
print, "POL_IMAGE_COMBINE is Done!"
end

