pro g_select_PSF_stars, Event, npsf_stars, image, x, y, mags, s_mags, ng, $
						xborder, yborder, FWHM_PSF, $
						id_PSF, clean_PSF_flag, npsf_stars_found, $
						n_candidate, grid_fwhm, ab, local_debug
;
;
;	selects PSF stars for ng x ng grid of image
;
;	A. rejects bright stars with neighbors inside of the
;		following table
;
;		for clean_PSF = 0:
;			diff_mag		distance
;			------------------------
;			0-1			5 * FWHM_PSF
;			1-2			4 * FWHM_PSF
;
;		for clean_PSF = 1:
;			diff_mag		distance
;			------------------------
;			0-1			3 * FWHM_PSF
;			1-2			4 * FWHM_PSF
;
;	B. also rejects stars with bright star flag set (s_mag = > 1 mag)
;
;	C. checks to be sure no PSF stars are within 12 FWHM of any other PSF star
;
;	D. and rejects any star with deleted pixels within 5 FHWMs
;
;	E. and does gaussian fit to trial PSF star, rejecting any with FWHMs too small or too wide
;
;	F. Ascertains whether the zone is "rich" in having > 100 bright stars, or "sparse"
;
;	G. If sparse, selects PSF stars from brightest down to either max_psf or mag_limit, which
;		ever comes first
;
;	H. If rich, finds first mag window with max_psf stars
;
;	returns id_PSF (ng,ng,50) with ID numbers of stars
;
;	DPC	20080624
;		20080716	modified rejection criteria to include table above, float FWHM
;		20090225	modified to allow clean_PSF options
;		20090303	modified to include rejection of PSF stars that are too close to each other
;		20090307	added gaussian fit test
;		20090405	imported all stars, must separate good from bad, only good can become PSF stars, all can negate
;		20090529	added sharpness, roundness remeasure to find, reject galaxy cores
;		20091209	added removal of bad star mags
;
;-----------------------------------------------------------
;
WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_Struct
w_ptr = PTR_Struct.PTR_wid_id
;
clean_out_galaxies = 1			; do cleaning of galaxies from stars
;
axis_ratio_limit = 2.00		; limit of a/b ratio for gaussian fits (was 4/3)
;
;	separate good stars
;
ind_PSF_candidate = indgen(n_candidate)
;
;debug = local_debug
debug = 0
;
if(debug gt 0) then print, "G_SELECT_PSF_STARS: incoming number of candidates = ",n_candidate,$
	" among a total of ",N_ELEMENTS(x)," stars"
if(debug ge 3) then begin
	for ic = 0, n_candidate-1 do begin
		print,"Candidate PSF star ",ic," has (x,y,mag) = ",x[ic],y[ic],mags[ic]
	endfor
endif
;
if(n_candidate le 0) then begin
	print_text, Event, "G_SELECT_PSF_STARS: ERROR - no good PSF star candidates available"
	goto, the_end
endif
;
msize = SIZE(mags)
if(msize[0] eq 2) then begin
	mags_here = REFORM(mags[0,ind_PSF_candidate])
	smags_here = REFORM(s_mags[0,ind_PSF_candidate])
	mags_all = REFORM(mags[0,*])
	smags_all = REFORM(s_mags[0,*])
endif else begin
	mags_here = mags[ind_PSF_candidate]
	smags_here = s_mags[ind_PSF_candidate]
	mags_all = mags
	smags_all = s_mags
endelse
;
x_can = x[ind_PSF_candidate]
y_can = y[ind_PSF_candidate]
;
;---------------------
;	add test, removal of bogus magnitudes - PSF candidates, first
;
ind_good = WHERE(mags_here gt -5.0 and mags_here lt 25.0,n_good)
if(n_good gt 0) then begin
	mags_here = mags_here[ind_good]
	smags_here = smags_here[ind_good]
	x_can = x_can[ind_good]
	y_can = y_can[ind_good]
endif else begin
	;
	;	no good PSF candidates in list, exit with status flag set
	;
	npsf_stars_found = 0
	goto, the_end
endelse
;
;	same test for all magnitudes
;
ind_good = WHERE(mags_all gt -5.0 and mags_all lt 25.0,n_good)
if(n_good gt 0) then begin
	mags_all = mags_all[ind_good]
	smags_all = smags_all[ind_good]
	x_all = x[ind_good]
	y_all = y[ind_good]
endif else begin
	;
	;	no good PSF candidates in list, exit with status flag set
	;
	npsf_stars_found = 0
	goto, the_end
endelse
;
xgwidth = x_can * 0.0		; to save widths for cleaning
ygwidth = xgwidth
;
PSF_neighbor_limit = 8.5 * FWHM_PSF		; so as to not overlap when building PSF
;
if(clean_PSF_flag eq 0) then begin
	mag_test = [1.0, 2.0, 3.0]
	rad_test = [6.0, 5.0, 3.0]
endif else begin
	mag_test = [-10.0, -6.0, -2.0, -1.0, 0.0, 1.0, 2.0, 3.0, 4.0]
	rad_test = [ 13.0, 12.0, 8.0, 6.0, 4.5, 3.5, 2.5, 1.5, 0.5]
endelse
;
mag_test_max = 4.0
rad_test = rad_test * FWHM_PSF
;
bad_pixel_value = -1.0e6
bad_pixel_dist = 3.0 * FWHM_PSF		; replace bad pixels within 6 FWHMs; also gauss fit range
;
min_nPSF = npsf_stars
max_nPSF = npsf_stars				; maximum number of PSF stars to save per zone
;
narrow_limit = 0.233 * FWHM_PSF		; limit for gaussian sigma tests
;
; mag_limit = 3.5	; former value
mag_limit = 4.5		; new value	; maximum limit between brightest and faintest stars
							; to be used in the PSF fitting - this is *different*
							; than the test for nearest neighbors
							;
;
;	clear IDs
;
id_PSF = lonarr(ng,ng,100) - 99		; set all to -99
;
;	get image size
;
i_size = SIZE(image)
nx = i_size[1]
ny = i_size[2]
if(nx eq 1024) then coadd = 0 else if(nx gt 1024) then coadd = 1 else coadd = -1
;
case coadd of
	0: begin
		xborder = 4
		yborder = 30
	   end
	1: begin
;		xborder = CEIL((nx - 1024) / 2.) + 3		; use imported values
;		yborder = CEIL((ny-1026)/2.) + 3
	   end
   -1: begin
   		xborder = 0
   		yborder = 30
   	   end
endcase
;
;	test for "raw" type image
;
if(nx eq 1024 and ny eq 1026) then raw = 1 else raw = 0
;
n_all_stars = N_ELEMENTS(x_all)
;
;	parse image into ng x ng grid
;
nx_ng = FLOOR(nx / ng)
ny_ng = FLOOR(ny / ng)
;
grid_fwhm = fltarr(ng,ng)
;
nxmin = intarr(ng)
nxmax = intarr(ng)
nymin = intarr(ng)
nymax = intarr(ng)
;
for i=0,ng-1 do begin
	nxmin[i] = 0 + nx_ng * i
	nxmax[i] = nxmin[i] + nx_ng - 1
	;
	nymin[i] = 0 + ny_ng * i
	nymax[i] = nymin[i] + ny_ng - 1
endfor
nxmax[ng-1] = nx-1
nymax[ng-1] = ny-1
;
;	for each of the ng x ng zones...
;
nPSF = intarr(ng,ng)
;
istop = 0
;
for ix = 0, ng-1 do begin
	;
	xmin = nxmin[ix]
	xmax = nxmax[ix]
	if(ix eq 0) then xmin = xmin + xborder
	if(ix eq (ng-1)) then xmax = xmax - xborder
	;
	for iy = 0, ng-1 do begin
		;
		this_loop_debug = 0
		;
		debug_loop_start:
		;
		ymin = nymin[iy]
		ymax = nymax[iy]
		;
		;	add a little more top and bottom avoidance for the ski jump regions
		;	and left, right image buffer
		;
		if(iy eq 0) then ymin = ymin + yborder
		if(iy eq (ng-1)) then ymax = ymax - yborder
		;
		;	select stars within this zone (plus a little to allow looking for neighbors)
		;
		z_ind = WHERE(x_can ge (0 > (xmin-PSF_neighbor_limit)) and $
					  x_can le ((nx-1) < (xmax+PSF_neighbor_limit)) and $
					  y_can ge (0 > (ymin-PSF_neighbor_limit)) and $
					  y_can le ((ny-1) < (ymax+PSF_neighbor_limit)),z_count)
		if(z_count lt 1) then begin
			print_text, Event, "    Warning - in Select-PSF-Stars: for zone [x,y] = "+STRING(ix+1, iy+1)+$
				" no stars remain"
			goto, loop_end
		endif
		;
		if(debug gt 0) then print_text, Event, " In Zone [X,Y] = "+STRING(ix,iy)+$
										" number of PSF candidates = "+STRING(z_count)
		;
		;	order by brightness
		;
		sort_ind = SORT(mags_here[z_ind])
		;
		;	is the brightest star brighter than 1st mag?		20091026
		;
		if(mags_here[z_ind[sort_ind[0]]] lt 1.0) then begin
			print_text, Event, "Warning - brightest star has mag = "+STRING(mags_here[z_ind[sort_ind[0]]])
		endif
		;
		;	determine whether this zone is "rich" or "poor"
		;
		n_in_zone = N_ELEMENTS(sort_ind)
		if(n_in_zone gt 200) then rich = 1 else rich = 0
		;
		if(debug gt 0) then print_text, Event,"  This zone has Rich = "+STRING(rich)
		;
		;	build histogram of stellar brightnesses, by 1/4 mag steps
		;
		bin_size = 0.25 ;mag
		bin_min = 0.0 < (mags_here[z_ind[sort_ind[0]]] - 0.125)
		bin_max = 25.0
		nbins = ((bin_max - bin_min) / bin_size) + 1
		mag_histo = HISTOGRAM(mags_here[z_ind[sort_ind]],BINSIZE=bin_size,MIN=bin_min,MAX=bin_max)
		xbin = (findgen(nbins) + 0.5)*bin_size + bin_min
		;
		;	if rich:
		;
		;		starting from the bright end, find the first bin for which
		;		the number of potential PSF stars is 4x the minimum number
		;		for that bin and the bins within the mag_limit parameter.
		;
		;	if poor:
		;
		;		take stars, starting with the brightest down to mag_limit
		;		or 5x max_psf, whichever happens first
		;
		if(rich) then begin
			;
			;	start by computing the window_summed vector from the histogram
			;
			window_sum_vec = fltarr(nbins)
			;
			window_width = FLOOR(mag_limit / bin_size)
			;
			ww_max = 0.0
			ind_ww = 0L
			for ibin = 0, nbins-1 do begin
				ibmin = ibin
				ibmax = (nbins-1) < (ibin + window_width - 1)
				window_sum_vec[ibin] = TOTAL(mag_histo[ibmin:ibmax])
				ww_max = ww_max > window_sum_vec[ibin]
				if(ww_max eq window_sum_vec[ibin]) then ind_ww = ibin		; just reset max, save index
			endfor
			;
			;	now search from bright end with minpsf cut
			;
			ind_mag = WHERE(window_sum_vec ge 4*npsf_stars, mag_count)
			if(mag_count gt 0) then begin
				ind_ww = ind_mag[0]
			endif
			start_mag = xbin[ind_ww]-0.5*bin_size
			end_mag = start_mag + mag_limit
			;
			;	test for at least min number of stars
			;
			xnd_mag = WHERE(window_sum_vec ge npsf_stars, xmag_count)
			if(xmag_count eq 0) then begin
				;
				;	nope, use stars from bright end to mag_limit
				;
				first_ind = WHERE(window_sum_vec gt 0, first_count)
				if(first_count gt 0) then begin
					start_mag = xbin[first_ind[0]] - 0.5*bin_size
					end_mag = start_mag + mag_limit
				endif else begin
					print_text, Event, "G_SELECT_PSF_STARS: ERROR - no stars found for PSF selection"
					stop
				endelse
			endif
			;
			if(debug gt 1) then begin
				print, "for 4*npsf_stars = ",4*npsf_stars, ", start_mag, end_mag = ", start_mag, end_mag," n in band = ",window_sum_vec[ind_ww]
;				window, 24
;				plot, xbin, mag_histo, psym=10, xrange=[5,17], xstyle=1
	;			wait, 1
	;			cursor, xx, yy, /wait
			endif
		endif else begin
			;
			;	"Poor" case
			;
			start_mag = mags_here[z_ind[sort_ind[0]]] - 0.01
			npoor = 1
			end_mag = -1
			for i = 1, n_in_zone-1 do begin
				dmag = mags_here[z_ind[sort_ind[i]]] - start_mag
				if(dmag le mag_limit or end_mag eq -1) then begin
					end_mag = mags_here[z_ind[sort_ind[i]]] + 0.01
					npoor++
					if(npoor ge 4*npsf_stars) then begin
						goto, poor_fin
					endif
				endif else if(end_mag ne -1) then goto, poor_fin	; extended to next bin
			endfor
			;
			poor_fin:
			if(debug gt 1) then begin
				print, "for ",npoor," stars, start_mag, end_mag = ", start_mag, end_mag
;				window, 24
;				plot, xbin, mag_histo, psym=10, xrange=[5,17], xstyle=1
;				cursor, xx, yy, /wait
;				wait, 0.5
			endif
		endelse
		;
		n_stars = N_ELEMENTS(sort_ind)
		n_stars50 = n_stars < 100
		;
		;	for each PSF candidate star, look for any found star within the rad_PSF and
		;	within 5 mag of this star
		;
		n_good = 0L
		;
		this_mag_dif = mag_limit	; to start
		;
		nloop = 0
		restart:
		;
		;	clear the counter
		;
		nPSF[ix,iy] = 0
		;
		brightest = 25.0			; mag of brightest PSF candidate passing all tests
		;
		if(debug gt 0) then print_text,Event," starting PSF candidate examination with "+$
									STRING(n_stars50)+" candidates"
		;
		n_try = 0L
		;
		for istar = 0, n_stars50-1 do begin
			;
			this_index = z_ind[sort_ind[istar]]
			this_x = x_can[this_index]
			this_y = y_can[this_index]
			this_mag = mags_here[this_index]
			;
;			print, "PSF Candidate number ",istar," has X,Y,Mag = ",this_x,this_y,this_mag
			;
			;	examine only the candidate stars between the histogram based mag limits
			;
			if(this_mag lt start_mag) then begin
				if(debug gt 1) then print, "Star ",istar," rejected as too bright"
				goto, outer_loop_end		; to try next star
			endif
			;
			if(this_mag gt end_mag) then begin
				if(debug gt 1) then print, "Star ",istar," rejected as too faint"
				goto, mag_limit_end		; no more stars in magnitude band
			endif
			;
			;	and between the x,y zonal limits
			;
			if(this_x lt xmin or this_x gt xmax or this_y lt ymin or this_y gt ymax) then begin
				if(debug ge 1) then print, "Star ",istar," not in zone - x,y = ",this_x,this_y
				goto, outer_loop_end
			endif
			;
			;	test every other star for being too close and too bright
			;
			for jstar = 0, n_all_stars-1 do begin
				;
				test_x = x_all[jstar]
				test_y = y_all[jstar]
				;
				dx = abs(test_x - this_x)
				dy = abs(test_y - this_y)
				rad = sqrt(dx * dx + dy * dy)
				;
				;	get mag dif
				;
				test_mag = mags_all[jstar]
				;
				if(abs(abs(test_mag)-99.9) lt 0.2) then test_mag = -5.0		; force deleted stars to bright
				;
				dmag = test_mag - this_mag
				if(dmag ge mag_test_max) then begin
;					if(debug gt 1) then print, "Star ",istar," rejected showing mag difference (",dmag,") from current starg as too large"
					goto, star_loop_end
				endif
				;
				;	test for the same star
				;
				if(abs(rad) lt 0.01) then begin
;					if(debug gt 1) then print, "Star ",istar," rejected as matching to itself"
					goto, star_loop_end
				endif
				;
				;	apply rejection table at top.
				;
				ind_reject = WHERE(dmag lt mag_test[*],reject_count)
				if(reject_count eq 0) then begin
					print_text, Event, "ERROR - dmag and mag_test make no sense. dmag = "+STRING(dmag)+$
								" mag_test = "+STRING(mag_test)
					stop
				endif
				;
				iz = ind_reject[0]		; zone with correct diff mag
				;
				;	radius test
				;
				if(rad gt rad_test[iz]) then begin
;					if(debug gt 1) then print, "Star ",istar," kept as too close to another"
					goto, star_loop_end	; stars adequately separated
				endif
				;
				;	ok, if got to here then we have a nearest
				;	neighbor that is too close and too bright,
				;	so don't include this star in the PSF list
				;
				if(debug gt 0) then begin
					print, "have a nearest neighbor star (",jstar,") that is too close, too bright - rejecting candidate PSF star = ",istar
					print, " PSF candate X, Y = ",this_x, this_y," Neighbor X, Y = ",test_x, test_y
					print, " PSF candidate mag = ",this_mag, " Neighbor mag = ",mags_all[jstar]
				endif
				;
				goto, outer_loop_end
				;
				star_loop_end:
				;
			endfor
			;
			n_try++
			;
			;	if got to here, then there were no problems and
			;	this star could be added to the PSF list
			;
			;	next, test for any bad pixels in the image within a guard
			;	region of this star
			;
			;--------- DPC - 20090330 - turned off bad pix test, as it is in calling program
			;
			xb0 = 0 > (this_x - bad_pixel_dist)
			xb1 = (nx-1) < (this_x + bad_pixel_dist)
			yb0 = 0 > (this_y - bad_pixel_dist)
			yb1 = (ny-1) < (this_y + bad_pixel_dist)
			;
			sub_image = image[xb0:xb1,yb0:yb1]
			vec = REFORM(sub_image[*,*])
			zz = median_filtered_mean(vec)
			;
			ind_bad_pixels = WHERE(sub_image eq bad_pixel_value, nbad_pix)
			;
			if(nbad_pix gt 0) then sub_image[ind_bad_pixels] = zz[0]
			;
;			if(nbad_pix gt 0) then begin
;				if(debug gt 0) then print, "Trial PSF star at x,y,mag = ",this_x, this_y, this_mag," had nearby bad pixels, so skipping"
;				goto, outer_loop_end
;			endif
			;
			;	and try gaussian fitting, as well
			;
			xg_test = this_x - xb0
			yg_test = this_y - yb0
			nxgauss = xb1 - xb0 + 1
			nygauss = yb1 - yb0 + 1
			;
			gfit = gauss2dfit(sub_image,coeffs,/TILT)
			basis = 'none'
			;
			if(debug ge 2) then begin
				print, "Star ",istar,"  Background Level = ",coeffs[0],"  Peak Fit value = ",coeffs[1]
				print, "X, Y widths (FWHM) = ",coeffs[2]*2.3548,coeffs[3]*2.3548,"  X, Y centers = ",this_x,this_y
				;print, "Position Angle = ",coeffs[6]
			endif
			;
			;	reject any star with any coeff = -NaN
			;
			n_bad_fit = 0
			ind = WHERE(FINITE(coeffs[*]), count)
			if(count ne 7) then begin
				n_bad_fit++
				basis = 'NaNs present'
				goto, gauss_loop_end
			endif
			;
			if(coeffs[1] lt 0.0) then begin
				n_bad_fit++
				basis = 'Gain<0'
				goto, gauss_loop_end
			end
			;
			if(abs(coeffs[2]) ge 15.0 or coeffs[2] lt narrow_limit) then begin
				n_bad_fit++
				basis = 'X width too large or too small. Xsig = '+STRING(coeffs[2])
				goto, gauss_loop_end
			endif
			;
			if(abs(coeffs[3]) ge 15.0 or coeffs[3] lt narrow_limit) then begin
				n_bad_fit++
				basis = 'Y width too large or too small. Ysig = '+STRING(coeffs[3])
				goto, gauss_loop_end
			endif
			;
			if(coeffs[4] lt 0.0 or coeffs[4] gt nxgauss) then begin
				n_bad_fit++
				basis = 'X position outside subimage. X = '+STRING(coeffs[4])
				goto, gauss_loop_end
			endif
			;
			if(coeffs[5] lt 0.0 or coeffs[5] gt nygauss) then begin
				n_bad_fit++
				basis = 'Y position outside subimage. Y = '+STRING(coeffs[5])
				goto, gauss_loop_end
			endif
			;
			;	test for elongated stars (axis ratio > 4/3  - DPC 20100116, was 3:2, then 2:1)
			;
			big_axis = coeffs[2] > coeffs[3]
			small_axis = coeffs[2] < coeffs[3]
			axis_ratio = big_axis / small_axis
			;
			;			compare to the distribution of axis ratios measured for this image
			;
			if(abs(axis_ratio - (ab[0] > 1.0) )/(ab[1] > 0.1) gt 5.0) then begin
				n_bad_fit++
				basis = 'Axis ratio too high = '+STRING(axis_ratio)
				goto, gauss_loop_end
			endif
			;
			;
			gauss_loop_end:
			if(n_bad_fit gt 0 and debug gt 0) then begin
				print, "G_SELECT_PSF_STARS: rejecting star ",istar," basis = ",basis
				print, "   gain, X, Y, sigx, sigy, sky = ",coeffs[1],this_x, this_y, coeffs[2], coeffs[3], coeffs[0]
			endif
			;
			if(n_bad_fit gt 0) then goto, outer_loop_end
			;
			xgwidth[nPSF[ix,iy]] = coeffs[2]
			ygwidth[nPSF[ix,iy]] = coeffs[3]
			;
;			print, "Star ",nPSF[ix,iy]," gaussian fit as X,Y,sigx, sigy = ",this_x,this_y,coeffs[2],coeffs[3]
			;
			;	third, check to be sure the nearest PSF star
			;	isn't too close
			;
			if(nPSF[ix,iy] gt 0) then begin
				for ipsf = 0, nPSF[ix,iy]-1 do begin
					psfid = id_PSF[ix,iy,ipsf]
					xipsf = x_can[psfid]
					yipsf = y_can[psfid]
					magipsf = mags_here[psfid]
					;
					dist = sqrt((this_x - xipsf)^2 + (this_y - yipsf)^2)
					if(dist lt PSF_neighbor_limit) then begin
						if(debug gt 0) then begin
							print, "rejecting PSF star as too close to another PSF star"
							print, "  This star x, y, mag = ",this_x, this_y, this_mag
							print, "  PSF star number ",ipsf," with x, y, mag = ",xipsf, yipsf, magipsf
							print, "  computed distance = ",dist," with limit of ",PSF_neighbor_limit
						endif
						goto, outer_loop_end
					endif
				endfor
			endif
			;
			;		OK - save the star on the final psf list
			;
			id_PSF[ix,iy,nPSF[ix,iy]] = z_ind[sort_ind[istar]]
			nPSF[ix,iy]++
			;
			;	if this was the first one saved, reset the bright and faint limits accordingly
			;		20100430 - DPC
			;
			if(nPSF[ix,iy] eq 1) then begin
				bright_diff = this_mag - start_mag
				start_mag = this_mag - 0.01
				end_mag = end_mag + bright_diff
			endif
			;
			;	come here to skip inclusion of star on PSF list
			;
			outer_loop_end:
		endfor
		;
		mag_limit_end:
		;
		if(debug gt 0) then begin
			print_text, Event, "  number surviving nearest neighbor and mag cut tests = "+STRING(n_try)
			print_text, Event, "  after gaussian testing, number of surviving candidates = "+STRING(nPSF[ix,iy])

		endif
		;
		;	test for the case of no surviving PSF candidates. If so,
		;	set status and end
		;
		if(nPSF[ix,iy] eq 0) then goto, loop_end
		;
		;	20090529 - DPC
		;		clean out galaxies by looking for fat gaussian fits
		;
		;	20090527 - DPC
		;		new test. For all surviving candidates, re-"find" them to get
		;		sharpness and roundness, then apply stringent limits to reject
		;		galaxies
		;
		if(clean_out_galaxies) then begin
			;
			if(debug gt 0) then print_text, Event, "ready to reject galaxies, number of starting PSF star candidates = "+STRING(nPSF[ix,iy])
			;
			;	first, get mean x, y widths and sigmas
			;
			zz_xwidth = median_filtered_mean(xgwidth[0:(nPSF[ix,iy]-1)])
			zz_ywidth = median_filtered_mean(ygwidth[0:(nPSF[ix,iy]-1)])
			;
			;	reject wider cases (galaxies, SFRs)
			;
			ind_wide = WHERE((xgwidth[0:(nPSF[ix,iy]-1)] - zz_xwidth[0])/zz_xwidth[1] gt 3.0 or $
						(ygwidth[0:(nPSF[ix,iy]-1)] - zz_ywidth[0])/zz_ywidth[1] gt 3.0,n_wide)
			;
			if(n_wide gt 0) then begin
     			;
     			;	remove those "stars"
     			;
     			id_PSF[ix,iy,ind_wide] = -99
     			nPSF[ix,iy] = nPSF[ix,iy] - n_wide
     			;
     			;	and push the list down
     			;
     			id_better = WHERE(REFORM(id_PSF[ix,iy,*]) ne -99,n_better)
     			if(n_better gt 0) then begin
	     			id_PSF[ix,iy,0:(nPSF[ix,iy]-1)] = id_PSF[ix,iy,id_better]
	     			id_PSF[ix,iy,nPSF[ix,iy]:99] = -99
	     		endif
     			;
     			if(debug gt 0) then print_text,Event, "removed "+STRING(n_wide)+$
     						" non-stars from PSF build due to widths"
     		endif
     		;
     		;	save fwhm for this grid cell
     		;
     		grid_fwhm[ix,iy] = sqrt(zz_xwidth[0] * zz_ywidth[0]) * 2.354
     		;
     		;	next, do sharpness & roundness cuts
     		;
			xg = fltarr(nPSF[ix,iy])
			yg = xg
			;
			sharp_psf = xg
			round_psf = xg
			;
			psfid = REFORM(id_PSF[ix,iy,0:(nPSF[ix,iy]-1)])
			ixpsf = ROUND(x_can[psfid])
			iypsf = ROUND(y_can[psfid])
			;
			maxbox = 13
			radius = 0.637*FWHM_PSF > 2.001
			radsq = radius^2           ;Radius is 1.5 sigma
 			nhalf = fix(radius) < (maxbox-1)/2
 			;
 			nbox = 2*nhalf + 1 ;# of pixels in side of convolution box
 			middle = nhalf          ;Index of central pixel
			;
 			lastro = nx - nhalf
 			lastcl = ny - nhalf
 			sigsq = (FWHM_PSF/2.35482 )^2
 			mask = bytarr( nbox, nbox )   ;Mask identifies valid pixels in convolution box
 			c = fltarr( nbox, nbox )      ;c will contain Gaussian convolution kernel
			;
 			dd = indgen(nbox-1) + 0.5 - middle ;Constants need to compute ROUND
 			dd2 = dd^2
 			w = 1. - 0.5*(abs(dd)-0.5) / (middle-0.5)
 			ir = (nhalf-1) > 1
			;
 			row2 = (findgen(Nbox)-nhalf)^2
			;
		 	for i = 0, nhalf do begin
		    	temp = row2 + i^2
		    	c[0,nhalf-i] = temp
		        c[0,nhalf+i] = temp
		 	endfor
			;
		 	mask = fix(c LE radsq)     ;MASK is complementary to SKIP in Stetson's Fortran
		 	good = where( mask, pixels)  ;Value of c are now equal to distance to center
			;
 			c = c*mask
 			c[good] = exp(-0.5*c[good]/sigsq)  ;Make c into a gaussian kernel
 			sumc = total(c)
 			sumcsq = total(c^2) - sumc^2/pixels
 			sumc = sumc/pixels
 			c[good] = (c[good] - sumc)/sumcsq
 			c1 = exp(-0.5*row2/sigsq)
 			sumc1 = total(c1)/nbox
 			sumc1sq = total(c1^2) - sumc1
 			c1 = (c1-sumc1)/sumc1sq
 			sumc = total(w)                         ;Needed for centroid computation
			;
 			h = convol(float(image),c)    ;Convolve image with kernel "c"
			;
    		h[0:(nhalf-1),*] = 0
    		h[(nx-nhalf):(nx-1),*] = 0
    		h[*,0:(nhalf-1)] = 0
    		h[*,(ny-nhalf):(ny-1)] = 0
			;
 			mask[middle,middle] = 0    ;From now on we exclude the central pixel
 			pixels = pixels -1      ;so the number of valid pixels is reduced by 1
 			good = where(mask)      ;"good" identifies position of valid pixels
 			xx= (good mod nbox) - middle   ;x and y coordinate of valid pixels
 			yy = fix(good/nbox) - middle    ;relative to the center
 			offset = yy*nx + xx
			for i = 0, (nPSF[ix,iy]-1) do begin
				temp = float(image[ixpsf[i]-nhalf:ixpsf[i]+nhalf,iypsf[i]-nhalf:iypsf[i]+nhalf])
     			d = h[ixpsf[i],iypsf[i]]                  ;"d" is actual pixel intensity
				;
				;  Compute Sharpness statistic
				;
     			sharp1 = (temp[middle,middle] - (total(mask*temp))/pixels)/d
     			sharp_psf[i] = sharp1
     			;
				;   Compute Roundness statistic
				;
     			dx = total( total(temp,2)*c1)
     			dy = total( total(temp,1)*c1)
     			around = 2*(dx-dy) / ( dx + dy )    ;Roundness statistic
     			round_psf[i] = around
     			;
;     			print, "Star ",i," has sharp, round = ",sharp_psf[i],round_psf[i]
     		endfor
     		;
     		;	find deviations
     		;
     		zzr = median_filtered_mean(round_psf[0:(nPSF[ix,iy]-1)])
     		zzs = median_filtered_mean(sharp_psf[0:(nPSF[ix,iy]-1)])
     		;
;     		debug = 1
;     		if(debug gt 1) then begin
;	     		window, 16
;	     		plot, sharp_psf, round_psf,psym=2,symsize=1.25,xtitle='sharpness',$
;	     			ytitle='roundness',xrange=[0,1],yrange=[-0.75,0.5]
;	     		;
;	     		;	roundlim = [-0.9,0.65]
;				;	sharplim = [0.2,0.65]
;				;
;	     		xx = [zzs[0]-zzs[1],zzs[0]+zzs[1]]
;	     		yy = [zzr[0],zzr[0]]
;	     		oplot, xx, yy
;	     		xx1 = [zzs[0],zzs[0]]
;	     		yy1 = [zzr[0]-zzr[1],zzr[0]+zzr[1]]
;	     		oplot, xx1, yy1
;	     		;
;	     		cursor, xxc,yyx,/wait
;	     		wait,0.3
;	     	endif
     		;
     		threshold = 2.5						; in sigmas
     		;
     		excursion = sqrt(((round_psf[0:(nPSF[ix,iy]-1)]-zzr[0])/zzr[1])^2 + $
     				((sharp_psf[0:(nPSF[ix,iy]-1)]-zzs[0])/zzs[1])^2)
     		;
     		ind_rs = WHERE(excursion gt threshold,bad_rs)
     		;
     		if(bad_rs gt 0) then begin
     			;
     			;	remove those stars
     			;
     			id_PSF[ix,iy,ind_rs] = -99
     			nPSF[ix,iy] = nPSF[ix,iy] - N_ELEMENTS(ind_rs)
     			;
     			;	and plot them
     			;
     			if(debug gt 1) then begin
	     			for ib = 0, N_ELEMENTS(ind_rs)-1 do begin
	     				xx2 = sharp_psf[ind_rs[ib]]
	     				yy2 = round_psf[ind_rs[ib]]
	     				;
;	     				plots, xx2, yy2, psym=6, symsize=1.6
	     			endfor
;	     			cursor, xx, yy, /wait
;	     			wait, 0.3
	     		endif
;	     		debug = 0
     			;
     			;	and push down
     			;
     			id_better = WHERE(REFORM(id_PSF[ix,iy,*]) ne -99,n_better)
     			if(n_better gt 0) then begin
	     			id_PSF[ix,iy,0:(nPSF[ix,iy]-1)] = id_PSF[ix,iy,id_better]
	     			id_PSF[ix,iy,nPSF[ix,iy]:99] = -99
	     		endif
     			;
     			if(debug gt 0) then print_text,Event, "removed "+STRING(bad_rs)+$
     								" non-stars from PSF build by roundess/sharpness"
     		endif
     		;
     	endif
     	;
     	if(debug gt 0) then begin
	     	print_text, Event, "  after cleaning out galaxies, number of surviving PSF candidates = "+STRING(nPSF[ix,iy])
	     	print_text, Event, "  FWHM for this grid cell = "+STRING(grid_fwhm[ix,iy])
	    endif
		;
		;	if still too many stars, take the brighest ones
		;
			if(nPSF[ix,iy] gt max_nPSF) then begin
				if(debug gt 0) then print, "Have ",nPSF[ix,iy]," candidate PSF stars, still. Will zone and brightness rank"
				if(debug ge 3) then begin
					for ic = 0, nPSF[ix,iy]-1 do begin
						ind_star = id_PSF[ix,iy,ic]
						print, "Candiate PSF star number ",ic," has (x,y,mag) = ",x_can[ind_star],y_can[ind_star],mags_here[ind_star]
					endfor
				endif
				;
				;	if ng = 1 or 2, then subzone
				;
				if(ng le 2) then begin
					;
					;	start by assigning subzone (0-8) for each star
					;
					sub_zone_x_width = (xmax - xmin)/3.
					sub_zone_y_width = (ymax - ymin)/3.
					;
					;	classify subzone for each star
					;
					sub_num = intarr( nPSF[ix,iy] )
					sub_id = intarr( nPSF[ix,iy] )
					sub_mag = fltarr( nPSF[ix,iy] )
					;
					for istar = 0, (nPSF[ix,iy]-1) do begin
						xsub = FLOOR((x[id_PSF[ix,iy,istar]] - xmin) / sub_zone_x_width)
						ysub = FLOOR((y[id_PSF[ix,iy,istar]] - ymin) / sub_zone_y_width)
						;
						xsub = (0 > xsub) < 2
						ysub = (0 > ysub) < 2
						;
						sub_num[istar] = ysub * 3 + xsub
						sub_mag[istar] = mags_here[id_PSF[ix,iy,istar]]
						;
					endfor
					;
					;	for each subzone, pick the brightest PSF star
					;
					keep_num = intarr( nPSF[ix,iy] )
					nkeep = 0
					;
					loop = 0
					;
					WHILE (nkeep lt max_nPSF and nkeep lt nPSF[ix,iy] and loop lt 10) do begin
						for isubzone = 0, 8 do begin
							ind_s = WHERE(sub_num eq isubzone, nsub)
							if(nsub gt 0) then begin
								;
								;	sort these by magnitudes
								;
								ind_sort = SORT( sub_mag[ind_s] )
								;
								;	and take the brightest
								;
								keep_num[nkeep] = id_PSF[ix,iy,ind_s[ind_sort[0]]]
								nkeep++
								;
								;	and remove that star from future searches
								;
								sub_num[ind_s[ind_sort[0]]] = -1
							endif
						endfor
						;
						loop++			; just in case
						;
					ENDWHILE
					;
					if(debug ge 3) then begin
						print, "number of subzone PSF stars kept = ",nkeep," on loop ",loop
						for ik = 0,nkeep-1 do begin
							ind_k = keep_num[ik]
							print, "Subzone PSF star ",ik," has (x,y,mag) = ",x_can[ind_k],y_can[ind_k],mags_here[ind_k]
						endfor
					endif
					;
					;	put the newly sorted list back into the index list
					;
					if(nkeep gt 0) then begin
						for ik = 0, nkeep-1 do begin
							id_PSF[ix,iy,ik] = keep_num[ik]
						endfor
						;
						for ik = nkeep, (nPSF[ix,iy]-1) do begin
							id_PSF[ix,iy,ik] = -99
						endfor
						nPSF[ix,iy] = nkeep
					endif else begin
						print_text, Event, "ERROR - nkeep = 0 ; bad sorting of PSF stars. Stopping"
					endelse
;
;
	;				window, 27, xsize=600,ysize=601
	;				xx = indgen(1024)
	;				yy = indgen(1026)
	;				plot, xx, yy, /nodata, xrange=[0,1023],yrange=[0,1025],xstyle=1,ystyle=1
	;				plots, x[id_PSF[0,0,0:(nPSF[0,0]-1)]], y[id_PSF[0,0,0:(nPSF[0,0]-1)]], psym=2
					;
;					stop
					;
				endif else begin
					;
					;	if ng > 2, just brightness rank
					;
					for ip = max_nPSF, (nPSF[ix,iy]-1) do begin
						id_PSF[ix,iy,ip] = -99
					endfor
					nPSF[ix,iy] = max_nPSF
				endelse
			endif
;		endelse
		;
		;	this zone complete - print number of PSF stars
		;
		print_text, Event, "For zone [x,y] = "+STRING(ix, iy)+" number of good PSF stars = "+STRING(nPSF[ix,iy])
		if(debug ge 3) then begin
			for ic = 0, nPSF[ix,iy]-1 do begin
				ind_s = id_PSF[ix,iy,ic]
				print,"Final PSF star ",ic," has (x,y,mag) = ",x_can[ind_s],y_can[ind_s],mags_here[ind_s]
			endfor
		endif
		;
		loop_end:
		;--------------------------------- ********** ---------------------------
		if(nPSF[ix,iy] eq 0) then begin
			print, "No PSF stars in zone ix,iy = ",ix,iy
			if(this_loop_debug eq 0) then begin
				;
				;	first pass, turn on debug and restart
				;
				debug = 1
				istop++
				this_loop_debug++
				goto, debug_loop_start
			endif else begin
				;
				;	second pass, give up
				;
				debug = 0
			endelse
		endif
		;--------------------------------- ********** ---------------------------
	endfor
endfor
;
;	all good PSF stars found, return
;
npsf_stars_found = TOTAL(nPSF)
;
;	write region file of PSF star locations for this image
;
if(debug gt 2) then begin
	file_out = dialog_pickfile(Title="Regions File to save PSF stars into")
	openw, lu_out, file_out, /GET_LUN
	for ix = 0, ng-1 do begin
		for iy = 0, ng-1 do begin
;			printf, lu_out, "  "
;			printf, lu_out, "PSF stars for region ix,iy = ",ix,iy
;			printf, lu_out, "--------------------------------------------------"
;			printf, lu_out, " Star      X         Y        Mag"
			;
			printf, lu_out, 'global color=white font="helvetica 10 normal" select=1 highlite=1 edit=1 move=1 delete=1 include=1 fixed=0 source'
			printf, lu_out, 'physical'
   			;
   			if(nPSF[ix,iy] gt 0) then begin
				for ipsf = 0, nPSF[ix,iy]-1 do begin
	;				printf, lu_out, ipsf+1, x[id_PSF[ix,iy,ipsf]], y[id_PSF[ix,iy,ipsf]], mags_here[id_PSF[ix,iy,ipsf]], $
	;							format='(1x,i3,3x,f7.1,3x,f7.1,3x,f7.2)'
					printf, lu_out, x_all[id_PSF[ix,iy,ipsf]], y_all[id_PSF[ix,iy,ipsf]],format='("circle(",f7.1,",",f7.1,",20)")'
				endfor
			endif
		endfor
	endfor
	;
	FREE_LUN, lu_out
	;
;	stop
	;
endif
;
the_end:
;
;if(istop eq 1) then stop
;
if(debug gt 0) then print,"G_SELECT_PSF_STARS: leaving with a total of ",$
		npsf_stars_found," good PSF star candidates"
;
end
