pro g_sky, image, skymode, skysig, mfm_mean, mfm_stdev, CIRCLERAD=circlerad,SILENT=silent,$
				LLQ=llq
;+
; NAME:
;       G_SKY		;  SKY, modified for GPIPS
; PURPOSE:
;       Determine the sky level in an image using the the procedure MMM
; EXPLANATION:
;       Approximately 4000 uniformly spaced pixels are selected for the
;       computation.  Adapted from the DAOPHOT routine of the same name.
;	    GPIPS -> traps bad pixels (-1 million value)
;
; CALLING SEQUENCE:
;       SKY, image, [ skymode, skysig, CIRCLE = ,/SILENT ]
;
; INPUTS:
;       IMAGE - One or two dimensional array
;
; OPTIONAL OUTPUT ARRAYS:
;       SKYMODE - Scalar, giving the mode of the sky pixel values of the
;               array IMAGE, as determined by the procedure MMM.
;       SKYSIG -  Scalar, giving standard deviation of sky brightness
;
;	MFM_MEAN - Scalar, giving image mean from MFM routine
;       MFM_STDEV - Scalar, giving image standard deviation from MFM routine
;
; INPUT KEYWORD PARAMETERS:
;       CIRCLERAD - Use the keyword to have SKY only select pixels within
;               the specified pixel radius of the center of the image.  If
;               CIRCLERAD = 1, then the radius is set equal to half the image
;               width.   Useful when the data is restricted to a circular area
;               of the image.
;
;       /SILENT - If this keyword is supplied and non-zero, then SKY will not
;               display the sky value and sigma at the terminal
;
; PROCEDURE:
;       A regular grid of points, not exceeding 4000 in number, is extracted
;       from the image array.  The mode of these pixel values is determined
;       by the procedure MMM.
;
; PROCEDURE CALLS:
;       MMM
; REVISION HISTORY:
;       Written, W. Landsman   STX Co.            September, 1987
;       Changed INDGEN to LINDGEN                 January, 1994
;       Fixed display of # of points used         March, 1994
;       Converted to IDL V5.0   W. Landsman   September 1997
;-
 On_error,2              ;Return to caller
 ;
 maxsky = 10000L          ;Maximum # of pixels to be used in sky calculation

 if N_params() eq 0 then begin
        print,'Syntax - sky, image, [ skymode, skysig , CIRCLE = , /SILENT ]'
        return
 endif
 ;
 if(KEYWORD_SET(llq)) then llq = 1 else llq = 0		; if set, ignore llq (and ulq, also)
 ;
 s = size(image)
 ;
 if(llq eq 1) then begin
 	this_image = image[521:*,*]
 	s = size(this_image)
 endif else begin
 	this_image = image
 endelse
 ;
 case s[0] of                           ;Number of dimensions in array?

 1: begin                                       ; 1-d vector
    istep = long(s[1]/float(maxsky) + 0.5) > 1
    npts = s[1]/istep
    ilow = (s[1] - istep*npts)/2
    index = ilow + lindgen(npts)*istep           ;Modified Jan. 94
    end

 2: begin                                       ;2-D array
	    if keyword_set(CIRCLERAD) then begin
	       width = max([s[1],s[2]])
	       if circlerad EQ 1 then rad = width/2 else rad = long(circlerad)
	       npts = !PI*rad^2
	    endif else npts = s[4]
	    fractn = float(maxsky)/npts
	    istep = fix(0.5 + 1/sqrt(fractn)) > 1
	    ilow = (s[1] - istep*((s[1] - 2)/istep) -1) / 2
	    n_x = s[1] / istep
	    jstep = s[2] *n_x/maxsky + 1
	    jlow = (s[2] - jstep*((s[2] -2)/jstep) - 1) /2
	    n_y = s[2]/jstep
	    npts = n_x*n_y
	    yy  =  jlow + jstep*(lindgen(npts)/n_x)
	    xx =  ilow + istep*(lindgen(npts) mod n_x)
	    index = xx + s[1]*yy
	    if keyword_set(CIRCLERAD) then begin
	      pixrad = (xx-s[1]/2)^2 + (yy-s[2]/2)^2
	      good = where(pixrad lt rad^2,ngood)
	      if ngood GT 0 then index = index[good] else $
	          message,'ERROR - Too few pixels within specified circle radius'
	    endif
	    ;
	    ;	test for bad pixels for GPIPS
	    ;
	    good_pix_index = WHERE(this_image[index] gt -1.0e5, ngp_count)
	    if(ngp_count gt 0) then begin
	    	index = index[good_pix_index]
	    endif else begin
	    	message,'ERROR - Too many bad pixels'
	    endelse
   end

 else: message,'Input array (first parameter) must be 1 or 2 dimensional'

 endcase

 mmm, this_image[index], skymode, skysig

 ;
 ;	DPC  20081230 - this returns values much too high if the sky
 ; 					has extended structure or emission. Instead,
 ;					do a grid of smaller regions, computing the
 ;					local rms and then form the mean of that.
 ;
 ;		20110630 - changed to undersampling each test region
 ;
 this_size = size(this_image)
 nx = this_size[1]
 ny = this_size[2]
 ngrid = 9
 nx9 = ROUND(nx/ngrid)
 ny9 = ROUND(ny/ngrid)
 rms = fltarr(ngrid*ngrid)
 mean_sky = fltarr(ngrid*ngrid)
 n_rms = 0
 xysize = 30
 xystep = 3
 for xg = 0, ngrid-1 do begin
 	xcen = ROUND(nx9 * (xg + 0.5))
 	xmin = 0 > (xcen-xysize)
 	xmax = (nx-1) < (xcen + xysize)
 	for yg = 0, ngrid-1 do begin
 		ycen = ROUND(ny9 * (yg + 0.5))
 		ymin = 0 > (ycen-xysize)
 		ymax = (ny-1) < (ycen+xysize)
 		;
 		vec = REFORM(this_image[xmin:xmax:xystep,ymin:ymax:xystep])
 		zz = median_filtered_mean(vec)
 		;
 		rms[n_rms] = zz[1]
 		mean_sky[n_rms] = zz[0]
 		n_rms++
 	endfor
 endfor
 ;
;	but, remove any zones with deleted means or zero stdevs
;
ind_good = WHERE(mean_sky gt -1.0e5 and rms gt 0.0,ngood)
if(ngood gt 0) then begin
	rms = rms[ind_good]
	mean_sky = mean_sky[ind_good]
 	zz_rms = median_filtered_mean(rms)
 	zz_sky = median_filtered_mean(mean_sky)
 	;
 	mfm_mean = zz_sky[0]
 	mfm_stdev = zz_rms[0]
endif else begin
	mfm_mean = -1.0e6
	mfm_stdev = 2.0e6
endelse
;
if(mfm_mean lt -1.0e5 or mfm_stdev gt 1.0e6) then begin
	a = 0.0
	b = 1./a
	;
	;stop
endif
;
 skymode = float(skymode)
 skysig = float(skysig)
 if not keyword_set(SILENT) then begin
        print,'Number of points used to find sky = ',N_elements(index)
        print,'Sky mode for this frame = ',skymode
        print,'Sky standard deviation  = ',skysig

		print,'MFM mean, stdev = ', mfm_mean, mfm_stdev
		for i = 0, ngood-1 do begin
			print, "zone ",i," sky, rms = ",mean_sky[i],rms[i]
		endfor
 endif

 return
 end
