function gauss_test, image, sky_mean, sky_sigma, x, y, fwhm, $
			x_fit, y_fit, fwhm_fit, BASIS=basis, debug
;
;	tests for whether a particular star in an image shows
;	a gaussian profile with "about" the right properties
;
;	returns total flux if yes, "0" if no (BASIS says why)
;
;	inputs: image is the input image
;			x, y are the nominal centers (floating point)
;			fwhm is the nominal fwhm
;
;	DPC	20090124 cut out from g_get_fwhm.pro
;		20100410 added ameoba
;		20100422 added peak test
;
;------------------------------------------------------------------
;	extract sub array
;
status = 0
basis = 'good'
image_size = SIZE(image)
nx = image_size[1]
ny = image_size[2]
;
box_size = CEIL(3.0 * fwhm)
border = 2
;
xmin = border > (ROUND(x) - box_size)
xmax = (nx-border) < (ROUND(x) + box_size)
ymin = (3*border) > (ROUND(y) - box_size)
ymax = (ny-(3*border)) < (ROUND(y) + box_size)
nsx = xmax - xmin + 1
nsy = ymax - ymin + 1
;
;	likely x,y peak location
;
x_p = ROUND(x) - xmin
y_p = ROUND(y) - ymin
;
if(nsx lt 5 or nsy lt 5 or x_p lt 1 or y_p lt 1 or x_p ge (nsx-1) or y_p ge (nsy-1)) then begin
	basis = 'Border'
	goto, the_exit
endif
;
sub_array = image[xmin:xmax, ymin:ymax]
;
;	clean any bad pixels
;
sub_array = fix_bad_pixels(sub_array)
mask_array = intarr(nsx,nsy)
;
;		select only those pixels brighter than 10% of the peak
;		(over the background) and contiguous with the peak pixels
;
;		only look over center 3x3 pixel zone
;
pk = MAX(sub_array[(x_p - 1):(x_p + 1),(y_p - 1):(y_p + 1)],xy_ind)
;
x_s_p = xy_ind MOD 3
y_s_p = FLOOR(xy_ind / 3)
;
x_pk = x_p - 1 + x_s_p
y_pk = y_p - 1 + y_s_p
;
;	initial background estimate
;
back_0 = median_filtered_mean(sub_array)
sb_array = sub_array - back_0[0]
;
;	move out from that point, turning on neighboring points that meet the cutoff.
;
ptest = 0.1 * (pk - back_0[0])
ind_10 = WHERE(sb_array ge ptest, n_10)
n_10 >= 10						; list is at least 10 elements long
pk_list = intarr(n_10,2)				; x, y positions
n_pk_list = 0
pk_list[n_pk_list,0] = x_pk
pk_list[n_pk_list,1] = y_pk
mask_array[x_pk,y_pk] = 1				; 1 for > test; 0 for not tested; -1 for less than test value
n_pk_list++								; points to next list position
;
;	print,'starting While loop. subarray size = ',nsx,nsy,' pk = ',pk
;
WHILE n_pk_list gt 0 do begin
	;
	;	test all neighbors that haven't already been tested
	;
	x_0 = pk_list[0,0]
	y_0 = pk_list[0,1]
	;
	x_1 = (x_0 - 1) > 0
	x_2 = (x_0 + 1) < (nsx - 1)
	;
	y_1 = (y_0 - 1) > 0
	y_2 = (y_0 + 1) < (nsy - 1)
	;
	;		y cut first
	;
	this_y = [y_1,y_2]
	for iy = 0, 1 do begin
		if(mask_array[x_0, this_y[iy]] eq 0) then begin
			if(sb_array[x_0, this_y[iy]] ge ptest) then begin
				mask_array[x_0, this_y[iy]] = 1
				pk_list[n_pk_list,0] = x_0
				pk_list[n_pk_list,1] = this_y[iy]
				n_pk_list++
			endif else mask_array[x_0, this_y[iy]] = -1
		endif
	endfor
	;
	;		x cut next
	;
	this_x = [x_1,x_2]
	for ix = 0, 1 do begin
		if(mask_array[this_x[ix], y_0] eq 0) then begin
			if(sb_array[this_x[ix], y_0] ge ptest) then begin
				mask_array[this_x[ix], y_0] = 1
				pk_list[n_pk_list,0] = this_x[ix]
				pk_list[n_pk_list,1] = y_0
				n_pk_list++
			endif else mask_array[this_x[ix], y_0] = -1
		endif
	endfor
	;
	;	this point checked, remove it from the list and push others down
	;
	pk_list[0:(n_10-2),*] = pk_list[1:(n_10-1),*]
	pk_list[(n_10-1),*] = -1
	n_pk_list--
	;
	;	go test next one
	;
ENDWHILE
;
;	determine background values
;
back = sub_array * ((1 - mask_array) < 1)
;
zz_back = median_filtered_mean(back)
;
;	load mean background into all pixels below 10* of peak
;
sub_array = sub_array * (mask_array > 0) + ((1 - mask_array) < 1) * zz_back[0]
;
;	stop
;
xs = indgen(nsx) + xmin
ys = indgen(nsy) + ymin
;
;	gaussian fit this star
;
gfit = gauss2dfit(sub_array,coeffs,xs,ys,/TILT)
;
;	reject any star with any coeff = -NaN
;
ind = WHERE(FINITE(coeffs[*]), count)
if(count ne 7) then begin
	basis = 'NaNs'
	goto, the_exit
endif
;
if(coeffs[1] lt 5.0*sky_sigma) then begin
	basis = 'Peak'
	goto, the_exit
endif
;
if(abs(coeffs[2]) ge (2.0*fwhm) or coeffs[2] lt 0.5) then begin
	basis = 'X-width'
	goto, the_exit
endif
;
if(abs(coeffs[3]) ge (2.0*fwhm) or coeffs[3] lt 0.5) then begin
	basis = 'Y-width'
	goto, the_exit
endif
;
if(coeffs[0] le (sky_mean - 3.0*sky_sigma)) then begin
	basis = 'Background'
	goto, the_exit
endif
;
if(coeffs[4] lt 0.0 or coeffs[4] gt float(nx)) then begin
	basis = 'X-center'
	goto, the_exit
endif
;
if(coeffs[5] lt 0.0 or coeffs[5] gt float(ny)) then begin
	basis = 'Y-center'
	goto, the_exit
endif
;
harm_fwhm = 2.3548 * sqrt(coeffs[2]*coeffs[3])
if(FINITE(harm_fwhm)) then begin
		;
		axis_ratio = (coeffs[2] > coeffs[3]) / (coeffs[2] < coeffs[3])
		;
		if(harm_fwhm ge 1.38) then begin
			if(axis_ratio lt 2.5) then begin
				fwhm_fit = harm_fwhm
				x_fit = coeffs[4]
				y_fit = coeffs[5]
				;
				if(abs(fwhm - fwhm_fit) / fwhm lt 0.60) then begin
					status = coeffs[1] * coeffs[2] * coeffs[3] * 2.0 * !PI
					if(debug ge 1) then begin
						print, "gauss_test: Background Level = ",coeffs[0]," Peak Fit value = ",coeffs[1]
						print, "gauss_test: X, Y widths (FWHM) = ",coeffs[2]*2.3548,coeffs[3]*2.3548," X, Y centers = ",coeffs[4],coeffs[5]
						;print, "Position Angle = ",coeffs[6]
					endif
				endif else begin
					basis = 'FWHM discrepancy'
					goto, the_exit
				endelse
			endif else begin
				basis = 'Axis Ratio'
				goto, the_exit
			endelse
		endif else begin
			basis = 'FWHM small'
			goto, the_exit
		endelse
endif else begin
	basis = 'FWHM NaN'
endelse
;
the_exit:
return, status
end
