function model_bad_pixels, input_array
;
;   uses GRID_TPS (thin plate spline) to interpolate
;   across bad pixels, using local pixels as predictors
;
;   DPC     20051223    original code
;           20051224    added trap for npts >= 7
;           20051231    added trap for npts = 0
;           20060103    upped ibad index to Longword type
;			20090527	also looks for NaN pixels
;
;			20120603	classifies bad pixels as 'edge', 'background', or 'star' and processes
;						each differently
;
;---------------------------------
;
;   get input array size - works for any size array
;
;print, "in model_bad_Pixels"
;
ns = SIZE(input_array)
nx = ns[1]
ny = ns[2]
;
;   create output array
;
output_array = input_array
;
bad_pix_test = -1.0e5
bad_pix_value = -1.0e6
;
;   locate all bad pixel values in input array
;
ind_bad = WHERE(input_array lt bad_pix_test or ~FINITE(input_array),nbad)
if(nbad eq 0) then begin
    ;
    ;  no bad pixels, return
    ;
    return, output_array
endif
;
;   interpret into set of (x,y) bad pixel pointers
;
ybad = fix(ind_bad / nx)
xbad = ind_bad - nx * ybad
;
;----------------------
;
;   for each bad pixel
;
ibad = 0L
for ibad = 0L, nbad-1L do begin
	;
	;	first, classify the type of bad pixel:
	;
	;	1. if within 5 rows of top or bottom, or 2 columns of left or right is edge, leave in place
	;	2. if pixel sits near or on local maximum, then is a star - gaussian fit
	;	3. if neither, then is background, use TPS method
	;
	if(ybad[ibad] le 5 or ybad[ibad] ge (ny-5) or xbad[ibad] le 2 or xbad[ibad] ge (nx-2)) then begin
		zvalue = bad_pix_value
	endif else begin
		;
		;	test for local maximum
		;
	    ;
	    ;  collect the nearest 31 pixels that have good data
	    ;
	    ix0 = 0 > (xbad[ibad]-5)
	    ix1 = (nx-1) < (xbad[ibad]+5)
	    iy0 = 0 > (ybad[ibad]-5)
	    iy1 = (ny-1) < (ybad[ibad]+5)
	    ;
	    inx = ix1 - ix0 + 1
	    iny = iy1 - iy0 + 1
	    ;
	    dist = fltarr(inx*iny)
	    ind = lonarr(inx*iny)
	    dvalue = fltarr(inx*iny)
	    ;
	    nd = 0
	    for ix = ix0, ix1 do begin
	       for iy = iy0, iy1 do begin
	         if(input_array[ix,iy] ne bad_pix_value and finite(input_array[ix,iy])) then begin
	          ind[nd] = iy*nx + ix
	          dist[nd] = sqrt((ix-xbad[ibad])^2 + (iy-ybad[ibad])^2)
	          dvalue[nd] = input_array[ix,iy]
	          nd++
	         endif
	       endfor
	    endfor
	    ;
	    ;   if nd is zero, then the entire region was deleted --
	    ;   set pixel value to deleted in this case.
	    ;
    	if(nd eq 0) then begin
    		 zvalue = bad_pix_value
    	endif else begin
    		;
        	ind = ind[0:nd-1]
        	dist = dist[0:nd-1]       ; trim to real values only
        	dvalue = dvalue[0:nd-1]
	        ;
	        ;  sort by dist
	        ;
	        ind_sort = SORT(dist)
	        ;
	        ;	and fit for decreasing slope with distance
	        ;
	        result = LINFIT(dist[ind_sort],dvalue[ind_sort],SIGMA=sigma)
	        ;
	        ;	if the slope is negative and strongly so (SNR > 5), we are on a star
	        ;
	        if(result[1] lt 0.0 and (abs(result[1])/sigma[1]) gt 5.0) then begin
	        	;
	        	;	this is a star
	        	;
;	        	print, 'Star found at (x,y) = ',xbad[ibad],ybad[ibad],' slope = ',result[1],' +/- ',sigma[1]
	        	;
	        	;	set up for a gaussian fit of the local values
	        	;
	        	;	have to go out to 5 - 8 HWs. If average FWHM is 3 pix, then need to go to +/- 9 pixels
	        	;
	        	dd = 9

				x0 = FLOOR(xbad[ibad]-dd) > 0
				x1 = CEIL(xbad[ibad]+dd) < (nx-1)
				y0 = FLOOR(ybad[ibad]-dd) > 0
				y1 = CEIL(ybad[ibad]+dd) < (ny-1)
				;
				ngx = x1 - x0 + 1
				ngy = y1 - y0 + 1
				xg = findgen(ngx) + x0
				yg = findgen(ngy) + y0
				;
				;	create 2D versions
				;
				MAKE_2D, xg, yg, xxg, yyg
				;
				xg = reform(xg)
				yg = reform(yg)
			    ;
	        	subimage = fix_bad_pixels(input_array[x0:x1, y0:y1])		; to enable fitting
	        	;
	        	;	for three passes of fitting, replacing central value
	        	;
	        	;	** looping added no value, so turned off
	        	;
	        	;for iloop = 0, 2 do begin
		        	;
		        	gfit = gauss2dfit(subimage,coeffs,xg,yg,/TILT)
					;
					debug = 0
					;
					if(debug gt 0) then begin
						print, "Background Level = ",coeffs[0]
						print, "Peak Fit value = ",coeffs[1]
						print, "X, Y widths (FWHM) = ",coeffs[2]*2.3548,coeffs[3]*2.3548
						print, "X, Y centers = ",coeffs[4],coeffs[5]
						print, "Position Angle = ",coeffs[6]
						;
			 			ymax = MAX(subimage)
						print, "Maximum pixel value = ",ymax
					endif
					;
					ind_x = WHERE(abs(xg - xbad[ibad]) lt 0.25)
					ind_y = WHERE(abs(yg - ybad[ibad]) lt 0.25)
					;
					zvalue = gfit[ind_x[0],ind_y[0]]
					;
					subimage[ind_x[0],ind_y[0]] = zvalue
					;
				;endfor
				;
;				print, 'X, Y indices in array = ',ind_x, ind_y,' zvalue = ',zvalue
				;
				;   TBD
				;
			endif else begin
				;
				;	was not a star, so do a TPS fit
				;
		        ;
		        ;  for the nearest up to 31 values...
		        ;
		        max_ind = 31 < nd
		        ;
		        ;   if there are not at least 7 points, replace pixel with deleted value
		        ;
		        if(max_ind ge 7) then begin
		            xpoint = intarr(max_ind)
		            ypoint = intarr(max_ind)
		            zpoint = dblarr(max_ind)
		            ;
		            for i = 0, max_ind-1 do begin
		                ;
		                ; unpack indices and load x, y, z vectors
		                ;
		                ypoint[i] = fix(ind[ind_sort[i]]/nx)
		                xpoint[i] = ind[ind_sort[i]] - nx * ypoint[i]
		                zpoint[i] = input_array[xpoint[i],ypoint[i]]
		            endfor
		            ;
		            ;  perform the TPS fit - return only the missing data value
		            ;
		            xs = xbad[ibad]
		            ys = ybad[ibad]
		            znew = GRID_TPS(xpoint, ypoint, zpoint, NGRID=2, START=[xs,ys], DELTA=1)
		            ;
		            ;  put this in the output array if it is within reason
		            ;
		            zvalue = znew[0,0]
		            ;
		            if(zvalue le 0.1*bad_pix_value) then zvalue = bad_pix_value
		            if(zvalue ge -0.1*bad_pix_value) then zvalue = bad_pix_value
        		endif else zvalue = bad_pix_value
        	endelse
        endelse
    endelse
    output_array[xbad[ibad],ybad[ibad]] = zvalue
endfor
;
;   all done, return output array
;
return, output_array
end



