function ppol_astrometry, Event, image_path, image, header, return_flag, platescale_est, tolerance,$
							find_scale, F17=f17, ARRAY_2MASS=array_2mass, TWO_MASS=two_mass, $
							FILE_2MASS=mass_array, DEFOCUS=defocus, LLQ=this_llq
;
;  Takes in an image and its header and returns a header with astrometry
;
;  MP  20081205  Original Version
;  DPC 20081220  Updated

;  Result flags
;  0 = no astrometry completed
;  1 = basic astrometry completed
;
;  99 = servers down, cannot get 2MASS information
;
;	2mass fetch may be skipped if the TWO_MASS flag is set (>0), then
;	the previous array may be fed back in to save time.
;
;	ARRAY_2MASS is the save array going back and forth - this is the successfully trimmed
;				array, saved after the current image astrometry completes
;
;	FILE_2MASS is the untrimmed array of data returned by the Visier call
;
COMMON saved_times, t0, skip_2mass		; save last skip_2mass value for reentry
;
t0 = systime(/seconds)
;
skip_2mass_flag = 0			; turn on for debugging on airplanes, only
;
final_header = header 		; in case of bad return
;
max_quad_count = 7
;
full_debug = 0
;
lots_flag = 0
;
sparse = 0
star_matches = 0
;
defocus_mode = KEYWORD_SET(DEFOCUS)
if(KEYWORD_SET(this_llq)) then this_llq = 1 else this_llq = 0	; if set, ignore llq info
;
orig_history = sxpar(header, 'HISTORY'); save it because it will be butchered
;
;	fetch 2mass stars, or use previous call's data
;
quad_criterion = 1						; initial try with mimir data requires stars in 3 of 4 quads
										; if all fails, relax this
this_tolerance = tolerance				; prepare to relax, if necessary
itol_loop = 0
i_lots_loop = 0
;
sat_limit = 11000L
;
;	check for coadds in this image
;
coadds = SXPAR(header, 'NUMCOADD', count=n_c_c)
if(n_c_c eq 0 or coadds eq 0) then coadds = 1
;
sat_limit *= coadds		; bump up by coadd number
;
do_2mass_call = 0
;
retry:
;
if(N_ELEMENTS(TWO_MASS) ne 0) then begin
	;
	;	keyword is present, test it
	;
	if(NOT(two_mass)) then do_2mass_call = 1 else do_2mass_call = 0
endif else do_2mass_call = 1
;
if(do_2mass_call) then begin
	;print, "fetching initial 2mass data"
	;
	find_scale = 11.0
	;
	if(skip_2mass_flag eq 0) then begin
		;
		;	need to fetch 2mass data
		;
		mass_array = ppol_fetch_mass(Event, header, f17); get 2MASS information if needed
		if(N_ELEMENTS(mass_array)/169.0 lt 0.75) then sparse = 1 else sparse = 0	; if < 0.75 per sq arcmin, sparse
		;
		;help, mass_array
		;
		array_test = size(mass_array, /type)
		if array_test ne 8 then begin; servers down
		  return_flag = 99
		  final_header = header
		  goto,skip_image
		endif
		;
		;	initial trimming (may be rerun if too many stars and need to move down
		;						2MASS list of stars)
		;
		skip_2mass = 0		; how many half-bins of brightness to skip down the 2MASS list
		;
		trimmed_mass_array = ppol_trim_mass_array(Event, mass_array, header, max_quad_count, skip_2mass, sparse)
		;
		;	save results
		;
		array_2mass = trimmed_mass_array
	endif
	;
endif else begin
	;
	;	requesting to use previous array, reload it
	;
	trimmed_mass_array = array_2mass
endelse
;
bad_pixel_fix_flag = 0 				; default is to not keep cleaned image after initial star finding
									; but for sparse fields, this is relaxed
refind:
;
idebug = 1 * full_debug
star_list = ppol_find_stars(Event, image_path, image, header, best_fwhm, max_quad_count, sat_limit, $
							find_scale, i_lots_o_stars, bad_pixel_fix_flag, idebug, nstars_found, $
							defocus_mode, this_llq)	; clean image, get fwhm, find stars
							;
trimmed_star_list = star_list
;
;	check that star finding didn't fail (returns single status flag)
;
single_star_flag = 0
nn = N_ELEMENTS(star_list)
if(nn eq 1) then begin
	;
	;	program gets here if the image fails to pass quality tests
	;
	if(star_list lt 0) then begin
		SXADDPAR, final_header, "HISTORY", 'This image judged unusable by S3/PPOL_ASTROMETRY'
;
		star_matches = 0
		status = -1
		goto, end_tests
	endif else begin
		;
		;	if we have a non-neg incoming status value, something is wrong
		print_test, Event, "ERROR - in ppol_astrometry - impossible error condition. Stopping"
		stop
	endelse
endif else begin
	;
	;	also test for presence of a single star (DPC 20091207)
	;
	if(nn eq 4) then begin
		single_star_flag = 1
	endif
endelse
;
nstars = size(star_list)
nstars = nstars[2]
;
if(idebug or single_star_flag) then print_text, Event, "PPOL_FIND_STARS found "+STRING(nstars,format='(I)')+" stars"
xsize = sxpar(header, 'NAXIS1')
ysize = sxpar(header, 'NAXIS2')
;
rematch:
;
idebug = 1 * full_debug
if(single_star_flag ne 1) then begin
	;
	idebug = 0
	;
	poss_matches = ppol_find_solutions(Event, this_tolerance, header, trimmed_mass_array, trimmed_star_list, $
									quad_criterion, STATUS=find_status, idebug, image_path)							; do the grunt work of finding triangles
	;
	; if return status was -1, don't do next check, but do try to reset quad_criterion if not reset already
	;
	if(find_status eq 0) then begin
		;
		idebug = 1 * full_debug
;		idebug = 1
		final_header = ppol_check_solutions(Event, this_tolerance, poss_matches, header, platescale_est, $
										save_this, N_MATCHED_STARS=n_matched_stars,$
										STAR_MATCHES=star_matches, STATUS=pcs_status, idebug)
		cs_status = sxpar(final_header, 'GPCOORD')							; a good status here = 1
		status = cs_status
	endif else begin
		star_matches = 0
		status = 0
		cs_status=-99
	endelse
	;
	next_try:
	;
	if status eq 0 then begin		; did not find any solutions examine flags to determine strategies
	  ;
	  ;	 if lots 'o stars, try moving 2mass window down some
	  ;
	  if(i_lots_o_stars) then begin
	  	if(i_lots_loop lt 8) then begin
	  		;
	  		print_text, Event, "Lots of stars, but no matches - moving 2MASS window down another half step"
	  		skip_2mass++
	  		i_lots_loop++
	  		;
	  		trimmed_mass_array = ppol_trim_mass_array(Event, mass_array, header, max_quad_count, skip_2mass, sparse)
			;
			;	save results
			;
			if(N_ELEMENTS(trimmed_mass_array) le 4) then begin	; ran out of 2mass stars
				skip_2mass = 0
				two_mass = 1
				trimmed_mass_array = ppol_trim_mass_array(Event, mass_array, header, max_quad_count, skip_2mass, sparse)
				goto, more_tries
			endif
			;
			array_2mass = trimmed_mass_array
	  		goto, rematch
	  	endif
	  	;
	  	;	still no match
	  	;
	  	more_tries:
	  	if(lots_flag eq 0) then begin
	  		print_text, Event, "Resetting the 2MASS window to top and restarting"
	  		lots_flag = 1			; but only do this once
	  		skip_2mass = 0
	  		i_lots_loop = 0
	  		trimmed_mass_array = ppol_trim_mass_array(Event, mass_array, header, max_quad_count, skip_2mass, sparse)
			;
			;	save results
			;
			if(N_ELEMENTS(trimmed_mass_array) le 4) then begin	; no 2mass stars, even after reset of window - this is bad
				;
				;	try to refind 2mass stars
				;
				mass_array = ppol_fetch_mass(Event, header, f17); get 2MASS information if needed
				if(N_ELEMENTS(mass_array)/144.0 lt 10.0) then sparse = 1 else sparse = 0
				array_test = size(mass_array, /type)
				if array_test ne 8 then begin; servers down
				  return_flag = 99
				  final_header = header
				  goto,skip_image
				endif
				skip_2mass = 0		; how many half-bins of brightness to skip down the 2MASS list
				trimmed_mass_array = ppol_trim_mass_array(Event, mass_array, header, max_quad_count, skip_2mass, sparse)
				if(N_ELEMENTS(trimmed_mass_array) le 4) then begin
					print_text, Event, "ERROR - failed to find any 2mass stars in field. skipping this image"
					goto, skip_image
				endif
				array_2mass = trimmed_mass_array
				;
				goto, refind
			endif
			array_2mass = trimmed_mass_array
	  		goto, rematch
	  	endif
	  	;
	  	print_text, Event, "Lots of stars, no matches after 2MASS window moved maximum"
	  	;
	  	print_text, Event, "Resetting the 2MASS window to top and restarting"
  		lots_flag = 0
  		skip_2mass = 0
  		i_lots_loop = 0
  		trimmed_mass_array = ppol_trim_mass_array(Event, mass_array, header, max_quad_count, skip_2mass, sparse)
		;
;	  	print_text, Event, "  - this is REALLY bad, so quitting astrometry for this image"
;	  	status = -1
;	  	goto, end_tests
	  endif
	  ;
	  ;	next, relax quad criterion
	  ;
	  if(quad_criterion eq 1) then begin			; try turning off mimir quad requirement
	    	quad_criterion = 0
	    	print_text, Event, "Warning - astrometry failed, turning off quad criterion"
	    	;
	    	goto, rematch
	  endif
	  ;
	  ;	next, try upping tolerance a tad
	  ;
	  if(itol_loop lt (2+bad_pixel_fix_flag) ) then begin
		this_tolerance = this_tolerance * 2.0
		print_text, Event, "Warning - astrometry failed, increasing tolerance to "+STRING(this_tolerance)
		;
		itol_loop++
		goto, rematch
	  endif
	  ;
	  ;	next, try keeping the bad pixel fixed image for processing, reset other loops
	  ;
	  bad_try:
	  ;
	  if(bad_pixel_fix_flag eq 0) then begin
		  bad_pixel_fix_flag = 1
		  itol_loop = 0
		  this_tolerance = tolerance
		  quad_criterion = 1
		  print_text, Event, "Warning - astrometry failed, keeping bad-pixel-fixed image for processing"
		  goto, refind
	  endif
	  ;
	  print_text, Event, "Failed N-star astrometry - trying 2-star version"
	  ;
	  ;	if we got here, then normal triangle fitting isn't working - try two star (minimal) solution
	  ;
	  final_header = ppol_2star(Event, this_tolerance, header, trimmed_mass_array, trimmed_star_list, $
						  platescale_est, STATUS=status, idebug)
	  ;
	  if(status ne -1) then begin
	  	star_matches = 2
	  	print_text, Event, "Completed 2-star astrometry OK"
	  	status = 12
	  	goto, end_tests
	  endif else print_text, Event, "Failed 2-star astrometry"
	  ;
	  ;	ok, even two star failed. How about 1 star?
	  ;
	  final_header = ppol_1star(Event, this_tolerance, header, trimmed_mass_array, trimmed_star_list, $
						  platescale_est, STATUS=status, idebug)
						  ;
	  if(status ne -1) then begin
	  	star_matches = 1
	  	print_text, Event, "Completed 1-star astrometry OK"
	  	status = 11
	  	goto, end_tests
	  endif else print_text, Event, "Failed 1-star astrometry"
	  ;
	  ;	all right, throw up your hands and surrender...
	  ;
	  print_text, Event, " Sparse star list, and all astrometry tries failed - this is BAD "
	  star_matches = 0
	  status = -1
	  ;
	endif
endif else begin
	;
	;	test for whether bad pixels have been fixed and the fixed image is being used. If not yet,
	;		then do so and refind.
	;
	if(bad_pixel_fix_flag eq 0) then begin
		  bad_pixel_fix_flag = 1
		  itol_loop = 0
		  this_tolerance = tolerance
		  quad_criterion = 1
		  print_text, Event, "Warning - astrometry failed, keeping bad-pixel-fixed image for processing"
		  goto, refind
	 endif
	;
	;	another possible single star fit (DPC 20091207)
	;
	final_header = ppol_1star(Event, this_tolerance, header, trimmed_mass_array, trimmed_star_list, $
						  platescale_est, STATUS=status, idebug)
						  ;
	  if(status ne -1) then begin
	  	star_matches = 1
	  	print_text, Event, "Completed 1-star astrometry OK"
	  	status = 11
	  	goto, end_tests
	  endif else print_text, Event, "Failed 1-star astrometry"
	  ;
	  ;	all right, throw up your hands and surrender...
	  ;
	  print_text, Event, " Sparse star list, and all astrometry tries failed - Bad Image - turning off USE flag "
	  star_matches = 0
	  status = -1
endelse
;
end_tests:
;
return_flag = status
;print, "After refine header, success flag = ",return_flag
;;
skip_image:
;
;	update header with number of stars found
;
SXADDPAR, final_header, "PPOLNSTR", star_matches,"NStars matched by PPOL Step 3"
;
return,final_header

end
