function s10_interpolate, xi, yi, x_vec, y_vec, z_vec, sz_vec
;
;	performs 2D interpolation with distance and uncertainty weighting
;
;	assumes calling program has already selected the *_vec entries
;		from a larger set of data
;
;	xi, yi are the location to compute new value for
;
;	x_vec, y_vec are the vectors of positions of z data location
;	z_vec are the existing data
;	sz_vec are the uncertainties
;
;-----------------------------------------------------------------------
;
order = 0
radial_power = 0
;
;	compute weights as (1/dist)^2 * (1/wt)^2, for dist > d_min,
;	where dist is measured in units of the stdev (d_min) of the
;	x, y data in these vectors.
;
;	inside of d_min, the first term is turned off.
;
zz_x = median_filtered_mean(x_vec)
zz_y = median_filtered_mean(y_vec)
;
if(FINITE(zz_x[1])) then rms_x = zz_x[1] else rms_x = 10.0
if(FINITE(zz_y[1])) then rms_y = zz_y[1] else rms_y = 10.0
;
d_min = (sqrt(rms_x^2 + rms_y^2)) < 90.0
d_min >= 30.0							; trap at no smaller than 30 pixels
;
d = sqrt(float(xi - x_vec)^2 + float(yi - y_vec)^2)
ind = WHERE(d le d_min, n_close)
if(n_close gt 0) then begin
	d[ind] = d_min							; trap at no smaller than d_min
endif
;
wt = (d_min / d)^radial_power
wt *= (1./(sz_vec)^2)
;
n_pts = N_ELEMENTS(z_vec)
;
;	order of the interpolation depends on the number of points
;
;	if < 4 can only do weighted mean
;	if > 4, can do bilinear
;	if > 8, can do biquad
;
if(n_close lt 10 or order eq 0) then begin
	z_tot = 0.0d0
	wt_tot = 0.0d0
	for ipts = 0, n_pts-1 do begin
		this_wt = wt[ipts]
		z_tot += z_vec[ipts] * this_wt
		wt_tot += this_wt
	endfor
	;
	if(wt_tot gt 1.0e-5) then begin
		z_return = z_tot / wt_tot
		sz_return = 1.0/ sqrt(wt_tot)
		goto, the_end
	endif else begin
		z_return = 0.0
		sz_return = 10.0
		goto, the_end
	endelse
endif
;
if((n_pts lt 8 and order gt 1) or (order eq 1)) then begin
	;
	;	bilinear (sort of)
	;
	;	z = a0 + a1*dx + a2*dy
	;
	;	form arrays for matrix least squares
	;
	wz = wt * z_vec
	wxy_t = dblarr(n_pts,3)
	wxy = dblarr(3,n_pts)
	;
	for ipts = 0, n_pts-1 do begin
		;
		wxy_t[ipts,0] = wt[ipts]
		wxy_t[ipts,1] = wt[ipts] * (xi - x_vec[ipts])
		wxy_t[ipts,2] = wt[ipts] * (yi - y_vec[ipts])
		;
	endfor
	for ia = 0, 2 do begin
		wxy[ia,*] = wxy_t[*,ia]
	endfor
	;
	;	compress
	;
	wxytwxy = wxy_t ## wxy
	;
	;	and invert
	;
	wxytwxy_invert = INVERT(wxytwxy, /DOUBLE)
	;
	;	and solve
	;
	a = wxytwxy_invert ## (wxy_t ## wz)
	;
	;	and evaluate
	;
	z_return = a[0]		; since all other terms are wrt (xi, yi)
	sz_return = sqrt(wxytwxy_invert[0,0])
	;
	goto, the_end
	;
endif else begin
	if(order ge 2) then begin
		;
		;	biquad
		;
		;	z = a0 + a1*dx + a2*dx^2 + a3*dy + a4*dy^2 + a5*dx*dy
		;
		wz = wt * z_vec
		wxy_t = dblarr(n_pts,6)
		wxy = dblarr(6,n_pts)
		;
		for ipts = 0, n_pts-1 do begin
			;
			dx = xi - x_vec[ipts]
			dy = yi - y_vec[ipts]
			;
			wxy_t[ipts,0] = wt[ipts]
			wxy_t[ipts,1] = wt[ipts] * dx
			wxy_t[ipts,2] = wt[ipts] * dx * dx
			wxy_t[ipts,3] = wt[ipts] * dy
			wxy_t[ipts,4] = wt[ipts] * dy * dy
			wxy_t[ipts,5] = wt[ipts] * dx * dy
			;
		endfor
		for ia = 0, 5 do begin
			wxy[ia,*] = wxy_t[*,ia]
		endfor
		;
		;	compress
		;
		wxytwxy = wxy_t ## wxy
		;
		;	and invert
		;
		wxytwxy_invert = INVERT(wxytwxy, /DOUBLE)
		;
		;	and solve
		;
		a = wxytwxy_invert ## (wxy_t ## wz)
		;
		;	and evaluate
		;
		z_return = a[0]		; since all other terms are wrt (xi, yi)
		;
	endif
endelse
;
the_end:
;
return, [z_return, sz_return]
end